# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
import logging
import os

from google.cloud.logging_v2.resource import Resource
from google.cloud.logging_v2._helpers import retrieve_metadata_server

_GAE_SERVICE_ENV = "GAE_SERVICE"
_GAE_VERSION_ENV = "GAE_VERSION"
_GAE_INSTANCE_ENV = "GAE_INSTANCE"
_GAE_ENV_VARS = [_GAE_SERVICE_ENV, _GAE_VERSION_ENV, _GAE_INSTANCE_ENV]
"""Environment variables set in App Engine environment."""

_CLOUD_RUN_SERVICE_ID = "K_SERVICE"
_CLOUD_RUN_REVISION_ID = "K_REVISION"
_CLOUD_RUN_CONFIGURATION_ID = "K_CONFIGURATION"
_CLOUD_RUN_SERVICE_ENV_VARS = [
    _CLOUD_RUN_SERVICE_ID,
    _CLOUD_RUN_REVISION_ID,
    _CLOUD_RUN_CONFIGURATION_ID,
]
_CLOUD_RUN_JOB_ID = "CLOUD_RUN_JOB"
_CLOUD_RUN_EXECUTION_ID = "CLOUD_RUN_EXECUTION"
_CLOUD_RUN_TASK_INDEX = "CLOUD_RUN_TASK_INDEX"
_CLOUD_RUN_TASK_ATTEMPT = "CLOUD_RUN_TASK_ATTEMPT"
_CLOUD_RUN_JOB_ENV_VARS = [
    _CLOUD_RUN_JOB_ID,
    _CLOUD_RUN_EXECUTION_ID,
    _CLOUD_RUN_TASK_INDEX,
    _CLOUD_RUN_TASK_ATTEMPT,
]
"""Environment variables set in Cloud Run environment."""

_FUNCTION_TARGET = "FUNCTION_TARGET"
_FUNCTION_SIGNATURE = "FUNCTION_SIGNATURE_TYPE"
_FUNCTION_NAME = "FUNCTION_NAME"
_FUNCTION_REGION = "FUNCTION_REGION"
_FUNCTION_ENTRY = "ENTRY_POINT"
_FUNCTION_ENV_VARS = [_FUNCTION_TARGET, _FUNCTION_SIGNATURE, _CLOUD_RUN_SERVICE_ID]
_LEGACY_FUNCTION_ENV_VARS = [_FUNCTION_NAME, _FUNCTION_REGION, _FUNCTION_ENTRY]
"""Environment variables set in Cloud Functions environments."""


_REGION_ID = "instance/region"
_ZONE_ID = "instance/zone"
_GCE_INSTANCE_ID = "instance/id"
"""Attribute in metadata server for compute region and instance."""

_GKE_CLUSTER_NAME = "instance/attributes/cluster-name"
"""Attribute in metadata server when in GKE environment."""

_GKE_CLUSTER_LOCATION = "instance/attributes/cluster-location"
"""Attribute in metadata server when in GKE environment."""

_PROJECT_NAME = "project/project-id"
"""Attribute in metadata server when in GKE environment."""

_GAE_RESOURCE_TYPE = "gae_app"
"""Resource type for App Engine environment."""

_CLOUD_RUN_JOB_RESOURCE_TYPE = "cloud_run_job"
"""Resource type for Cloud Run Jobs."""

_GAE_TRACE_ID_LABEL = "appengine.googleapis.com/trace_id"
"""Extra trace label to be added on App Engine environments"""

_CLOUD_RUN_JOBS_EXECUTION_NAME_LABEL = "run.googleapis.com/execution_name"
_CLOUD_RUN_JOBS_TASK_INDEX_LABEL = "run.googleapis.com/task_index"
_CLOUD_RUN_JOBS_TASK_ATTEMPT_LABEL = "run.googleapis.com/task_attempt"
"""Extra labels for Cloud Run environments to be recognized by Cloud Run Jobs web UI."""


def _create_functions_resource():
    """Create a standardized Cloud Functions resource.
    Returns:
        google.cloud.logging.Resource
    """
    project = retrieve_metadata_server(_PROJECT_NAME)
    region = retrieve_metadata_server(_REGION_ID)
    if _FUNCTION_NAME in os.environ:
        function_name = os.environ.get(_FUNCTION_NAME)
    elif _CLOUD_RUN_SERVICE_ID in os.environ:
        function_name = os.environ.get(_CLOUD_RUN_SERVICE_ID)
    else:
        function_name = ""
    resource = Resource(
        type="cloud_function",
        labels={
            "project_id": project if project else "",
            "function_name": function_name if function_name else "",
            "region": region.split("/")[-1] if region else "",
        },
    )
    return resource


def _create_kubernetes_resource():
    """Create a standardized Kubernetes resource.
    Returns:
        google.cloud.logging.Resource
    """
    location = retrieve_metadata_server(_GKE_CLUSTER_LOCATION)
    cluster_name = retrieve_metadata_server(_GKE_CLUSTER_NAME)
    project = retrieve_metadata_server(_PROJECT_NAME)

    resource = Resource(
        type="k8s_container",
        labels={
            "project_id": project if project else "",
            "location": location if location else "",
            "cluster_name": cluster_name if cluster_name else "",
        },
    )
    return resource


def _create_compute_resource():
    """Create a standardized Compute Engine resource.
    Returns:
        google.cloud.logging.Resource
    """
    instance = retrieve_metadata_server(_GCE_INSTANCE_ID)
    zone = retrieve_metadata_server(_ZONE_ID)
    project = retrieve_metadata_server(_PROJECT_NAME)
    resource = Resource(
        type="gce_instance",
        labels={
            "project_id": project if project else "",
            "instance_id": instance if instance else "",
            "zone": zone if zone else "",
        },
    )
    return resource


def _create_cloud_run_service_resource():
    """Create a standardized Cloud Run service resource.
    Returns:
        google.cloud.logging.Resource
    """
    region = retrieve_metadata_server(_REGION_ID)
    project = retrieve_metadata_server(_PROJECT_NAME)
    resource = Resource(
        type="cloud_run_revision",
        labels={
            "project_id": project if project else "",
            "service_name": os.environ.get(_CLOUD_RUN_SERVICE_ID, ""),
            "revision_name": os.environ.get(_CLOUD_RUN_REVISION_ID, ""),
            "location": region.split("/")[-1] if region else "",
            "configuration_name": os.environ.get(_CLOUD_RUN_CONFIGURATION_ID, ""),
        },
    )
    return resource


def _create_cloud_run_job_resource():
    """Create a standardized Cloud Run job resource.
    Returns:
        google.cloud.logging.Resource
    """
    region = retrieve_metadata_server(_REGION_ID)
    project = retrieve_metadata_server(_PROJECT_NAME)
    resource = Resource(
        type=_CLOUD_RUN_JOB_RESOURCE_TYPE,
        labels={
            "project_id": project if project else "",
            "job_name": os.environ.get(_CLOUD_RUN_JOB_ID, ""),
            "location": region.split("/")[-1] if region else "",
        },
    )
    return resource


def _create_app_engine_resource():
    """Create a standardized App Engine resource.
    Returns:
        google.cloud.logging.Resource
    """
    zone = retrieve_metadata_server(_ZONE_ID)
    project = retrieve_metadata_server(_PROJECT_NAME)
    resource = Resource(
        type=_GAE_RESOURCE_TYPE,
        labels={
            "project_id": project if project else "",
            "module_id": os.environ.get(_GAE_SERVICE_ENV, ""),
            "version_id": os.environ.get(_GAE_VERSION_ENV, ""),
            "zone": zone if zone else "",
        },
    )
    return resource


def _create_global_resource(project):
    """Create a global resource.
    Args:
        project (str): The project ID to pass on to the resource
    Returns:
        google.cloud.logging.Resource
    """
    return Resource(type="global", labels={"project_id": project if project else ""})


def detect_resource(project=""):
    """Return the default monitored resource based on the local environment.
    If GCP resource not found, defaults to `global`.

    Args:
        project (str): The project ID to pass on to the resource (if needed)
    Returns:
        google.cloud.logging.Resource: The default resource based on the environment
    """
    gke_cluster_name = retrieve_metadata_server(_GKE_CLUSTER_NAME)
    gce_instance_name = retrieve_metadata_server(_GCE_INSTANCE_ID)

    if all([env in os.environ for env in _GAE_ENV_VARS]):
        # App Engine Flex or Standard
        return _create_app_engine_resource()
    elif gke_cluster_name is not None:
        # Kubernetes Engine
        return _create_kubernetes_resource()
    elif all([env in os.environ for env in _LEGACY_FUNCTION_ENV_VARS]) or all(
        [env in os.environ for env in _FUNCTION_ENV_VARS]
    ):
        # Cloud Functions
        return _create_functions_resource()
    elif all([env in os.environ for env in _CLOUD_RUN_SERVICE_ENV_VARS]):
        # Cloud Run
        return _create_cloud_run_service_resource()
    elif all([env in os.environ for env in _CLOUD_RUN_JOB_ENV_VARS]):
        # Cloud Run
        return _create_cloud_run_job_resource()
    elif gce_instance_name is not None:
        # Compute Engine
        return _create_compute_resource()
    else:
        # use generic global resource
        return _create_global_resource(project)


@functools.lru_cache(maxsize=None)
def _get_environmental_labels(resource_type):
    """Builds a dictionary of labels to be inserted into a LogRecord of the given resource type.
    This function should only build a dict of items that are consistent across multiple logging statements
    of the same resource type, such as environment variables. Th

    Returns:
        dict:
            A dict representation of labels and the values of those labels
    """
    labels = {}
    environ_vars = {
        _CLOUD_RUN_JOB_RESOURCE_TYPE: {
            _CLOUD_RUN_JOBS_EXECUTION_NAME_LABEL: _CLOUD_RUN_EXECUTION_ID,
            _CLOUD_RUN_JOBS_TASK_INDEX_LABEL: _CLOUD_RUN_TASK_INDEX,
            _CLOUD_RUN_JOBS_TASK_ATTEMPT_LABEL: _CLOUD_RUN_TASK_ATTEMPT,
        }
    }

    if resource_type in environ_vars:
        for key, env_var in environ_vars[resource_type].items():
            val = os.environ.get(env_var, "")
            if val:
                labels[key] = val

    return labels


def add_resource_labels(resource: Resource, record: logging.LogRecord):
    """Returns additional labels to be appended on to a LogRecord object based on the
    local environment. Defaults to an empty dictionary if none apply. This is only to be
    used for CloudLoggingHandler, as the structured logging daemon already does this.

    Args:
        resource (google.cloud.logging.Resource): Resource based on the environment
        record (logging.LogRecord): A LogRecord object representing a log record
    Returns:
        Dict[str, str]: New labels to append to the labels of the LogRecord
    """
    if not resource:
        return None

    # Get environmental labels from the resource type
    labels = _get_environmental_labels(resource.type)

    # Add labels from log record
    if resource.type == _GAE_RESOURCE_TYPE and record._trace is not None:
        labels[_GAE_TRACE_ID_LABEL] = record._trace

    return labels



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.protobuf import struct_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.aiplatform.v1beta1",
    manifest={
        "ModelMonitoringAlertCondition",
        "ModelMonitoringAnomaly",
        "ModelMonitoringAlert",
    },
)


class ModelMonitoringAlertCondition(proto.Message):
    r"""Monitoring alert triggered condition.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        threshold (float):
            A condition that compares a stats value
            against a threshold. Alert will be triggered if
            value above the threshold.

            This field is a member of `oneof`_ ``condition``.
    """

    threshold: float = proto.Field(
        proto.DOUBLE,
        number=1,
        oneof="condition",
    )


class ModelMonitoringAnomaly(proto.Message):
    r"""Represents a single model monitoring anomaly.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        tabular_anomaly (google.cloud.aiplatform_v1beta1.types.ModelMonitoringAnomaly.TabularAnomaly):
            Tabular anomaly.

            This field is a member of `oneof`_ ``anomaly``.
        model_monitoring_job (str):
            Model monitoring job resource name.
        algorithm (str):
            Algorithm used to calculated the metrics, eg:
            jensen_shannon_divergence, l_infinity.
    """

    class TabularAnomaly(proto.Message):
        r"""Tabular anomaly details.

        Attributes:
            anomaly_uri (str):
                Additional anomaly information. e.g. Google
                Cloud Storage uri.
            summary (str):
                Overview of this anomaly.
            anomaly (google.protobuf.struct_pb2.Value):
                Anomaly body.
            trigger_time (google.protobuf.timestamp_pb2.Timestamp):
                The time the anomaly was triggered.
            condition (google.cloud.aiplatform_v1beta1.types.ModelMonitoringAlertCondition):
                The alert condition associated with this
                anomaly.
        """

        anomaly_uri: str = proto.Field(
            proto.STRING,
            number=1,
        )
        summary: str = proto.Field(
            proto.STRING,
            number=2,
        )
        anomaly: struct_pb2.Value = proto.Field(
            proto.MESSAGE,
            number=3,
            message=struct_pb2.Value,
        )
        trigger_time: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=4,
            message=timestamp_pb2.Timestamp,
        )
        condition: "ModelMonitoringAlertCondition" = proto.Field(
            proto.MESSAGE,
            number=5,
            message="ModelMonitoringAlertCondition",
        )

    tabular_anomaly: TabularAnomaly = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="anomaly",
        message=TabularAnomaly,
    )
    model_monitoring_job: str = proto.Field(
        proto.STRING,
        number=2,
    )
    algorithm: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ModelMonitoringAlert(proto.Message):
    r"""Represents a single monitoring alert. This is currently used
    in the SearchModelMonitoringAlerts api, thus the alert wrapped
    in this message belongs to the resource asked in the request.

    Attributes:
        stats_name (str):
            The stats name.
        objective_type (str):
            One of the supported monitoring objectives:
            ``raw-feature-drift`` ``prediction-output-drift``
            ``feature-attribution``
        alert_time (google.protobuf.timestamp_pb2.Timestamp):
            Alert creation time.
        anomaly (google.cloud.aiplatform_v1beta1.types.ModelMonitoringAnomaly):
            Anomaly details.
    """

    stats_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    objective_type: str = proto.Field(
        proto.STRING,
        number=2,
    )
    alert_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    anomaly: "ModelMonitoringAnomaly" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="ModelMonitoringAnomaly",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import collections
from typing import Union, TYPE_CHECKING

from google.cloud.aiplatform import base
from vertexai import generative_models
from vertexai.preview.evaluation import _pre_eval_utils
from vertexai.preview.evaluation import constants
from vertexai.preview.evaluation import utils
from vertexai.preview.evaluation.metrics import (
    _base as metrics_base,
)
from vertexai.preview.evaluation.metrics import pairwise_metric
from vertexai.preview.evaluation.metrics import pointwise_metric

if TYPE_CHECKING:
    import pandas as pd

_DEFAULT_MODEL_NAME = "gemini-2.0-flash-001"
_LOGGER = base.Logger(__name__)


class RubricBasedMetric(metrics_base._Metric):
    """Config for Rubric-Based Eval."""

    def __init__(
        self,
        *,
        generation_config: metrics_base.RubricGenerationConfig,
        critique_metric: Union[
            pointwise_metric.PointwiseMetric, pairwise_metric.PairwiseMetric
        ]
    ):
        """Initializes RubricBasedMetric.

        Args:
          generation_config: Config for rubric generation.
          critique_metric: Pointwise/pairwise metric for rubric critique.
        """
        super().__init__(metric=critique_metric._metric)

        self.generation_config = generation_config
        self.critique_metric = critique_metric

    def generate_rubrics(
        self,
        eval_dataset: "pd.Dataframe",
    ) -> "pd.DataFrame":
        """Generates rubrics for given eval dataset."""
        if not self.generation_config.model:
            model = generative_models.GenerativeModel(model_name=_DEFAULT_MODEL_NAME)
        else:
            model = self.generation_config.model

        if constants.Dataset.RUBRICS_COLUMN in eval_dataset.columns:
            _LOGGER.warning(
                "Rubrics column already exists in the dataset. Skipping rubric"
                " generation."
            )
            return eval_dataset

        responses = _pre_eval_utils._generate_responses_from_gemini_model(
            model,
            eval_dataset,
            self.generation_config.prompt_template,
        )
        if self.generation_config.parsing_fn:
            parsing_fn = self.generation_config.parsing_fn
        else:
            parsing_fn = utils.parse_rubrics
        dataset_with_rubrics = eval_dataset.copy()
        aggregated = collections.defaultdict(list)
        for idx, response in enumerate(responses):
            result = parsing_fn(response)
            if isinstance(result, dict):
                questions = result.pop("questions", None)
                if questions is not None:
                    aggregated[constants.Dataset.RUBRICS_COLUMN].append(
                        (idx, questions)
                    )
                for key, value in result.items():
                    aggregated[key].append((idx, value))
            else:
                aggregated[constants.Dataset.RUBRICS_COLUMN].append((idx, result))
        for key, values in aggregated.items():
            dataset_with_rubrics[key] = None
            dataset_with_rubrics[key] = dataset_with_rubrics[key].astype(object)
            for idx, value in values:
                dataset_with_rubrics.at[idx, key] = value
        return dataset_with_rubrics



"""Main Streamlit application for the AI Business Consultant."""

import streamlit as st
import os
from config import config
from components.sidebar import sidebar
from components.chat_interface import chat_interface

# Page configuration
st.set_page_config(
    page_title=config.app_name,
    page_icon="🤖",
    layout="wide",
    initial_sidebar_state="expanded"
)

# Custom CSS for better styling
st.markdown("""
<style>
    .stChat > div {
        background-color: #f8f9fa;
        border-radius: 10px;
        padding: 10px;
        margin-bottom: 10px;
    }
    
    .stChatMessage {
        background-color: blue;
        border-radius: 8px;
        padding: 15px;
        margin: 10px 0;
        box-shadow: 0 2px 4px rgba(0,0,0,0.1);
    }
    
    .stExpander {
        background-color: #f8f9fa;
        border-radius: 8px;
        border: 1px solid #e9ecef;
        margin: 10px 0;
    }
    
    .insight-card {
        background-color: #f8f9fa;
        border-left: 4px solid #007bff;
        padding: 15px;
        margin: 10px 0;
        border-radius: 5px;
    }
    
    .recommendation-card {
        background-color: #f8f9fa;
        border-left: 4px solid #28a745;
        padding: 15px;
        margin: 10px 0;
        border-radius: 5px;
    }
    
    .risk-card {
        background-color: #fff3cd;
        border-left: 4px solid #ffc107;
        padding: 15px;
        margin: 10px 0;
        border-radius: 5px;
    }
    
    .error-card {
        background-color: #f8d7da;
        border-left: 4px solid #dc3545;
        padding: 15px;
        margin: 10px 0;
        border-radius: 5px;
    }
</style>
""", unsafe_allow_html=True)

def main():
    """Main application function."""
    
    # Check if API keys are configured
    if not config.openai_api_key or config.openai_api_key == "sk-placeholder":
        st.error("⚠️ OpenAI API key not configured. Please set up your .env file.")
        st.markdown("""
        **Setup Instructions:**
        1. Copy `.env.example` to `.env`
        2. Add your OpenAI API key to the `.env` file
        3. Restart the application
        
        **Required in .env file:**
        ```
        OPENAI_API_KEY=your_openai_api_key_here
        ```
        """)
        return
    
    # Render sidebar
    sidebar.render()
    
    # Main content area
    col1, col2 = st.columns([3, 1])
    
    with col1:
        # Header
        st.title(f"🤖 {config.agent_name}")
        st.markdown("*Your AI-powered business consultant for strategic insights and recommendations*")
        
        # Handle pending messages from sidebar
        if "pending_message" in st.session_state:
            pending_msg = st.session_state.pending_message
            del st.session_state.pending_message
            chat_interface.handle_user_input(pending_msg)
        
        # Show examples if requested
        if st.session_state.get("show_examples", False):
            st.markdown("### 💡 Example Questions")
            st.markdown("Click any example to start a conversation:")
            
            for i, example in enumerate(st.session_state.get("example_questions", [])):
                if st.button(f"📝 {example[:80]}{'...' if len(example) > 80 else ''}", key=f"example_{i}"):
                    chat_interface.handle_user_input(example)
                    st.session_state.show_examples = False
                    st.rerun()
            
            if st.button("❌ Close Examples"):
                st.session_state.show_examples = False
                st.rerun()
            
            st.markdown("---")
        
        # Chat interface
        chat_interface.render()
    
    with col2:
        # Quick stats or additional info
        st.markdown("### 📊 Quick Stats")
        
        # Display conversation stats
        message_count = len(st.session_state.get("messages", []))
        st.metric("Messages", message_count)
        
        # Display current analysis info
        if st.session_state.get("current_analysis"):
            analysis = st.session_state.current_analysis
            st.metric("Last Analysis", analysis.get("type", "None").replace("_", " ").title())
        
        # Usage tips
        st.markdown("### 💡 Tips")
        st.markdown("""
        - Be specific in your questions
        - Ask for market analysis, competitor research, or risk assessment
        - Request strategic recommendations
        - Use the sidebar for quick actions
        """)
        
        # Recent insights
        if st.session_state.get("current_analysis"):
            st.markdown("### 🔍 Recent Insights")
            analysis = st.session_state.current_analysis
            
            if analysis.get("type") == "market_analysis":
                insights = analysis.get("insights", [])
                if insights:
                    st.markdown(f"**{len(insights)} insights found**")
                    for insight in insights[:3]:  # Show first 3
                        st.markdown(f"• {insight['category']}")
            
            elif analysis.get("type") == "recommendations":
                recommendations = analysis.get("recommendations", [])
                if recommendations:
                    st.markdown(f"**{len(recommendations)} recommendations**")
                    for rec in recommendations[:3]:  # Show first 3
                        st.markdown(f"• {rec.get('category', 'Recommendation')}")

if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import inspect
import json
import pickle
import logging as std_logging
import warnings
from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union

from google.api_core import gapic_v1
from google.api_core import grpc_helpers_async
from google.api_core import exceptions as core_exceptions
from google.api_core import retry_async as retries
from google.api_core import operations_v1
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.protobuf.json_format import MessageToJson
import google.protobuf.message

import grpc  # type: ignore
import proto  # type: ignore
from grpc.experimental import aio  # type: ignore

from google.cloud.spanner_admin_instance_v1.types import spanner_instance_admin
from google.iam.v1 import iam_policy_pb2  # type: ignore
from google.iam.v1 import policy_pb2  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from .base import InstanceAdminTransport, DEFAULT_CLIENT_INFO
from .grpc import InstanceAdminGrpcTransport

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class _LoggingClientAIOInterceptor(
    grpc.aio.UnaryUnaryClientInterceptor
):  # pragma: NO COVER
    async def intercept_unary_unary(self, continuation, client_call_details, request):
        logging_enabled = CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        )
        if logging_enabled:  # pragma: NO COVER
            request_metadata = client_call_details.metadata
            if isinstance(request, proto.Message):
                request_payload = type(request).to_json(request)
            elif isinstance(request, google.protobuf.message.Message):
                request_payload = MessageToJson(request)
            else:
                request_payload = f"{type(request).__name__}: {pickle.dumps(request)}"

            request_metadata = {
                key: value.decode("utf-8") if isinstance(value, bytes) else value
                for key, value in request_metadata
            }
            grpc_request = {
                "payload": request_payload,
                "requestMethod": "grpc",
                "metadata": dict(request_metadata),
            }
            _LOGGER.debug(
                f"Sending request for {client_call_details.method}",
                extra={
                    "serviceName": "google.spanner.admin.instance.v1.InstanceAdmin",
                    "rpcName": str(client_call_details.method),
                    "request": grpc_request,
                    "metadata": grpc_request["metadata"],
                },
            )
        response = await continuation(client_call_details, request)
        if logging_enabled:  # pragma: NO COVER
            response_metadata = await response.trailing_metadata()
            # Convert gRPC metadata `<class 'grpc.aio._metadata.Metadata'>` to list of tuples
            metadata = (
                dict([(k, str(v)) for k, v in response_metadata])
                if response_metadata
                else None
            )
            result = await response
            if isinstance(result, proto.Message):
                response_payload = type(result).to_json(result)
            elif isinstance(result, google.protobuf.message.Message):
                response_payload = MessageToJson(result)
            else:
                response_payload = f"{type(result).__name__}: {pickle.dumps(result)}"
            grpc_response = {
                "payload": response_payload,
                "metadata": metadata,
                "status": "OK",
            }
            _LOGGER.debug(
                f"Received response to rpc {client_call_details.method}.",
                extra={
                    "serviceName": "google.spanner.admin.instance.v1.InstanceAdmin",
                    "rpcName": str(client_call_details.method),
                    "response": grpc_response,
                    "metadata": grpc_response["metadata"],
                },
            )
        return response


class InstanceAdminGrpcAsyncIOTransport(InstanceAdminTransport):
    """gRPC AsyncIO backend transport for InstanceAdmin.

    Cloud Spanner Instance Admin API

    The Cloud Spanner Instance Admin API can be used to create,
    delete, modify and list instances. Instances are dedicated Cloud
    Spanner serving and storage resources to be used by Cloud
    Spanner databases.

    Each instance has a "configuration", which dictates where the
    serving resources for the Cloud Spanner instance are located
    (e.g., US-central, Europe). Configurations are created by Google
    based on resource availability.

    Cloud Spanner billing is based on the instances that exist and
    their sizes. After an instance exists, there are no additional
    per-database or per-operation charges for use of the instance
    (though there may be additional network bandwidth charges).
    Instances offer isolation: problems with databases in one
    instance will not affect other instances. However, within an
    instance databases can affect each other. For example, if one
    database in an instance receives a lot of requests and consumes
    most of the instance resources, fewer resources are available
    for other databases in that instance, and their performance may
    suffer.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _grpc_channel: aio.Channel
    _stubs: Dict[str, Callable] = {}

    @classmethod
    def create_channel(
        cls,
        host: str = "spanner.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> aio.Channel:
        """Create and return a gRPC AsyncIO channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): Deprecated. A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`. This argument will be
                removed in the next major version of this library.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            aio.Channel: A gRPC AsyncIO channel object.
        """

        return grpc_helpers_async.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            default_scopes=cls.AUTH_SCOPES,
            scopes=scopes,
            default_host=cls.DEFAULT_HOST,
            **kwargs,
        )

    def __init__(
        self,
        *,
        host: str = "spanner.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        channel: Optional[Union[aio.Channel, Callable[..., aio.Channel]]] = None,
        api_mtls_endpoint: Optional[str] = None,
        client_cert_source: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        ssl_channel_credentials: Optional[grpc.ChannelCredentials] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'spanner.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if a ``channel`` instance is provided.
            credentials_file (Optional[str]): Deprecated. A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if a ``channel`` instance is provided.
                This argument will be removed in the next major version of this library.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            channel (Optional[Union[aio.Channel, Callable[..., aio.Channel]]]):
                A ``Channel`` instance through which to make calls, or a Callable
                that constructs and returns one. If set to None, ``self.create_channel``
                is used to create the channel. If a Callable is given, it will be called
                with the same arguments as used in ``self.create_channel``.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for the grpc channel. It is ignored if a ``channel`` instance is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure a mutual TLS channel. It is
                ignored if a ``channel`` instance or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}
        self._operations_client: Optional[operations_v1.OperationsAsyncClient] = None

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if isinstance(channel, aio.Channel):
            # Ignore credentials if a channel was passed.
            credentials = None
            self._ignore_credentials = True
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None
        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )

        if not self._grpc_channel:
            # initialize with the provided callable or the default channel
            channel_init = channel or type(self).create_channel
            self._grpc_channel = channel_init(
                self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                # Set ``credentials_file`` to ``None`` here as
                # the credentials that we saved earlier should be used.
                credentials_file=None,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                    ("grpc.keepalive_time_ms", 120000),
                ],
            )

        self._interceptor = _LoggingClientAIOInterceptor()
        self._grpc_channel._unary_unary_interceptors.append(self._interceptor)
        self._logged_channel = self._grpc_channel
        self._wrap_with_kind = (
            "kind" in inspect.signature(gapic_v1.method_async.wrap_method).parameters
        )
        # Wrap messages. This must be done after self._logged_channel exists
        self._prep_wrapped_messages(client_info)

    @property
    def grpc_channel(self) -> aio.Channel:
        """Create the channel designed to connect to this service.

        This property caches on the instance; repeated calls return
        the same channel.
        """
        # Return the channel from cache.
        return self._grpc_channel

    @property
    def operations_client(self) -> operations_v1.OperationsAsyncClient:
        """Create the client designed to process long-running operations.

        This property caches on the instance; repeated calls return the same
        client.
        """
        # Quick check: Only create a new client if we do not already have one.
        if self._operations_client is None:
            self._operations_client = operations_v1.OperationsAsyncClient(
                self._logged_channel
            )

        # Return the client from cache.
        return self._operations_client

    @property
    def list_instance_configs(
        self,
    ) -> Callable[
        [spanner_instance_admin.ListInstanceConfigsRequest],
        Awaitable[spanner_instance_admin.ListInstanceConfigsResponse],
    ]:
        r"""Return a callable for the list instance configs method over gRPC.

        Lists the supported instance configurations for a
        given project.
        Returns both Google-managed configurations and
        user-managed configurations.

        Returns:
            Callable[[~.ListInstanceConfigsRequest],
                    Awaitable[~.ListInstanceConfigsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_instance_configs" not in self._stubs:
            self._stubs["list_instance_configs"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/ListInstanceConfigs",
                request_serializer=spanner_instance_admin.ListInstanceConfigsRequest.serialize,
                response_deserializer=spanner_instance_admin.ListInstanceConfigsResponse.deserialize,
            )
        return self._stubs["list_instance_configs"]

    @property
    def get_instance_config(
        self,
    ) -> Callable[
        [spanner_instance_admin.GetInstanceConfigRequest],
        Awaitable[spanner_instance_admin.InstanceConfig],
    ]:
        r"""Return a callable for the get instance config method over gRPC.

        Gets information about a particular instance
        configuration.

        Returns:
            Callable[[~.GetInstanceConfigRequest],
                    Awaitable[~.InstanceConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_instance_config" not in self._stubs:
            self._stubs["get_instance_config"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/GetInstanceConfig",
                request_serializer=spanner_instance_admin.GetInstanceConfigRequest.serialize,
                response_deserializer=spanner_instance_admin.InstanceConfig.deserialize,
            )
        return self._stubs["get_instance_config"]

    @property
    def create_instance_config(
        self,
    ) -> Callable[
        [spanner_instance_admin.CreateInstanceConfigRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the create instance config method over gRPC.

        Creates an instance configuration and begins preparing it to be
        used. The returned long-running operation can be used to track
        the progress of preparing the new instance configuration. The
        instance configuration name is assigned by the caller. If the
        named instance configuration already exists,
        ``CreateInstanceConfig`` returns ``ALREADY_EXISTS``.

        Immediately after the request returns:

        - The instance configuration is readable via the API, with all
          requested attributes. The instance configuration's
          [reconciling][google.spanner.admin.instance.v1.InstanceConfig.reconciling]
          field is set to true. Its state is ``CREATING``.

        While the operation is pending:

        - Cancelling the operation renders the instance configuration
          immediately unreadable via the API.
        - Except for deleting the creating resource, all other attempts
          to modify the instance configuration are rejected.

        Upon completion of the returned operation:

        - Instances can be created using the instance configuration.
        - The instance configuration's
          [reconciling][google.spanner.admin.instance.v1.InstanceConfig.reconciling]
          field becomes false. Its state becomes ``READY``.

        The returned long-running operation will have a name of the
        format ``<instance_config_name>/operations/<operation_id>`` and
        can be used to track creation of the instance configuration. The
        metadata field type is
        [CreateInstanceConfigMetadata][google.spanner.admin.instance.v1.CreateInstanceConfigMetadata].
        The response field type is
        [InstanceConfig][google.spanner.admin.instance.v1.InstanceConfig],
        if successful.

        Authorization requires ``spanner.instanceConfigs.create``
        permission on the resource
        [parent][google.spanner.admin.instance.v1.CreateInstanceConfigRequest.parent].

        Returns:
            Callable[[~.CreateInstanceConfigRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_instance_config" not in self._stubs:
            self._stubs["create_instance_config"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/CreateInstanceConfig",
                request_serializer=spanner_instance_admin.CreateInstanceConfigRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_instance_config"]

    @property
    def update_instance_config(
        self,
    ) -> Callable[
        [spanner_instance_admin.UpdateInstanceConfigRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the update instance config method over gRPC.

        Updates an instance configuration. The returned long-running
        operation can be used to track the progress of updating the
        instance. If the named instance configuration does not exist,
        returns ``NOT_FOUND``.

        Only user-managed configurations can be updated.

        Immediately after the request returns:

        - The instance configuration's
          [reconciling][google.spanner.admin.instance.v1.InstanceConfig.reconciling]
          field is set to true.

        While the operation is pending:

        - Cancelling the operation sets its metadata's
          [cancel_time][google.spanner.admin.instance.v1.UpdateInstanceConfigMetadata.cancel_time].
          The operation is guaranteed to succeed at undoing all changes,
          after which point it terminates with a ``CANCELLED`` status.
        - All other attempts to modify the instance configuration are
          rejected.
        - Reading the instance configuration via the API continues to
          give the pre-request values.

        Upon completion of the returned operation:

        - Creating instances using the instance configuration uses the
          new values.
        - The new values of the instance configuration are readable via
          the API.
        - The instance configuration's
          [reconciling][google.spanner.admin.instance.v1.InstanceConfig.reconciling]
          field becomes false.

        The returned long-running operation will have a name of the
        format ``<instance_config_name>/operations/<operation_id>`` and
        can be used to track the instance configuration modification.
        The metadata field type is
        [UpdateInstanceConfigMetadata][google.spanner.admin.instance.v1.UpdateInstanceConfigMetadata].
        The response field type is
        [InstanceConfig][google.spanner.admin.instance.v1.InstanceConfig],
        if successful.

        Authorization requires ``spanner.instanceConfigs.update``
        permission on the resource
        [name][google.spanner.admin.instance.v1.InstanceConfig.name].

        Returns:
            Callable[[~.UpdateInstanceConfigRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_instance_config" not in self._stubs:
            self._stubs["update_instance_config"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/UpdateInstanceConfig",
                request_serializer=spanner_instance_admin.UpdateInstanceConfigRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["update_instance_config"]

    @property
    def delete_instance_config(
        self,
    ) -> Callable[
        [spanner_instance_admin.DeleteInstanceConfigRequest], Awaitable[empty_pb2.Empty]
    ]:
        r"""Return a callable for the delete instance config method over gRPC.

        Deletes the instance configuration. Deletion is only allowed
        when no instances are using the configuration. If any instances
        are using the configuration, returns ``FAILED_PRECONDITION``.

        Only user-managed configurations can be deleted.

        Authorization requires ``spanner.instanceConfigs.delete``
        permission on the resource
        [name][google.spanner.admin.instance.v1.InstanceConfig.name].

        Returns:
            Callable[[~.DeleteInstanceConfigRequest],
                    Awaitable[~.Empty]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_instance_config" not in self._stubs:
            self._stubs["delete_instance_config"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/DeleteInstanceConfig",
                request_serializer=spanner_instance_admin.DeleteInstanceConfigRequest.serialize,
                response_deserializer=empty_pb2.Empty.FromString,
            )
        return self._stubs["delete_instance_config"]

    @property
    def list_instance_config_operations(
        self,
    ) -> Callable[
        [spanner_instance_admin.ListInstanceConfigOperationsRequest],
        Awaitable[spanner_instance_admin.ListInstanceConfigOperationsResponse],
    ]:
        r"""Return a callable for the list instance config
        operations method over gRPC.

        Lists the user-managed instance configuration long-running
        operations in the given project. An instance configuration
        operation has a name of the form
        ``projects/<project>/instanceConfigs/<instance_config>/operations/<operation>``.
        The long-running operation metadata field type
        ``metadata.type_url`` describes the type of the metadata.
        Operations returned include those that have
        completed/failed/canceled within the last 7 days, and pending
        operations. Operations returned are ordered by
        ``operation.metadata.value.start_time`` in descending order
        starting from the most recently started operation.

        Returns:
            Callable[[~.ListInstanceConfigOperationsRequest],
                    Awaitable[~.ListInstanceConfigOperationsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_instance_config_operations" not in self._stubs:
            self._stubs[
                "list_instance_config_operations"
            ] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/ListInstanceConfigOperations",
                request_serializer=spanner_instance_admin.ListInstanceConfigOperationsRequest.serialize,
                response_deserializer=spanner_instance_admin.ListInstanceConfigOperationsResponse.deserialize,
            )
        return self._stubs["list_instance_config_operations"]

    @property
    def list_instances(
        self,
    ) -> Callable[
        [spanner_instance_admin.ListInstancesRequest],
        Awaitable[spanner_instance_admin.ListInstancesResponse],
    ]:
        r"""Return a callable for the list instances method over gRPC.

        Lists all instances in the given project.

        Returns:
            Callable[[~.ListInstancesRequest],
                    Awaitable[~.ListInstancesResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_instances" not in self._stubs:
            self._stubs["list_instances"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/ListInstances",
                request_serializer=spanner_instance_admin.ListInstancesRequest.serialize,
                response_deserializer=spanner_instance_admin.ListInstancesResponse.deserialize,
            )
        return self._stubs["list_instances"]

    @property
    def list_instance_partitions(
        self,
    ) -> Callable[
        [spanner_instance_admin.ListInstancePartitionsRequest],
        Awaitable[spanner_instance_admin.ListInstancePartitionsResponse],
    ]:
        r"""Return a callable for the list instance partitions method over gRPC.

        Lists all instance partitions for the given instance.

        Returns:
            Callable[[~.ListInstancePartitionsRequest],
                    Awaitable[~.ListInstancePartitionsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_instance_partitions" not in self._stubs:
            self._stubs["list_instance_partitions"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/ListInstancePartitions",
                request_serializer=spanner_instance_admin.ListInstancePartitionsRequest.serialize,
                response_deserializer=spanner_instance_admin.ListInstancePartitionsResponse.deserialize,
            )
        return self._stubs["list_instance_partitions"]

    @property
    def get_instance(
        self,
    ) -> Callable[
        [spanner_instance_admin.GetInstanceRequest],
        Awaitable[spanner_instance_admin.Instance],
    ]:
        r"""Return a callable for the get instance method over gRPC.

        Gets information about a particular instance.

        Returns:
            Callable[[~.GetInstanceRequest],
                    Awaitable[~.Instance]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_instance" not in self._stubs:
            self._stubs["get_instance"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/GetInstance",
                request_serializer=spanner_instance_admin.GetInstanceRequest.serialize,
                response_deserializer=spanner_instance_admin.Instance.deserialize,
            )
        return self._stubs["get_instance"]

    @property
    def create_instance(
        self,
    ) -> Callable[
        [spanner_instance_admin.CreateInstanceRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the create instance method over gRPC.

        Creates an instance and begins preparing it to begin serving.
        The returned long-running operation can be used to track the
        progress of preparing the new instance. The instance name is
        assigned by the caller. If the named instance already exists,
        ``CreateInstance`` returns ``ALREADY_EXISTS``.

        Immediately upon completion of this request:

        - The instance is readable via the API, with all requested
          attributes but no allocated resources. Its state is
          ``CREATING``.

        Until completion of the returned operation:

        - Cancelling the operation renders the instance immediately
          unreadable via the API.
        - The instance can be deleted.
        - All other attempts to modify the instance are rejected.

        Upon completion of the returned operation:

        - Billing for all successfully-allocated resources begins (some
          types may have lower than the requested levels).
        - Databases can be created in the instance.
        - The instance's allocated resource levels are readable via the
          API.
        - The instance's state becomes ``READY``.

        The returned long-running operation will have a name of the
        format ``<instance_name>/operations/<operation_id>`` and can be
        used to track creation of the instance. The metadata field type
        is
        [CreateInstanceMetadata][google.spanner.admin.instance.v1.CreateInstanceMetadata].
        The response field type is
        [Instance][google.spanner.admin.instance.v1.Instance], if
        successful.

        Returns:
            Callable[[~.CreateInstanceRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_instance" not in self._stubs:
            self._stubs["create_instance"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/CreateInstance",
                request_serializer=spanner_instance_admin.CreateInstanceRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_instance"]

    @property
    def update_instance(
        self,
    ) -> Callable[
        [spanner_instance_admin.UpdateInstanceRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the update instance method over gRPC.

        Updates an instance, and begins allocating or releasing
        resources as requested. The returned long-running operation can
        be used to track the progress of updating the instance. If the
        named instance does not exist, returns ``NOT_FOUND``.

        Immediately upon completion of this request:

        - For resource types for which a decrease in the instance's
          allocation has been requested, billing is based on the
          newly-requested level.

        Until completion of the returned operation:

        - Cancelling the operation sets its metadata's
          [cancel_time][google.spanner.admin.instance.v1.UpdateInstanceMetadata.cancel_time],
          and begins restoring resources to their pre-request values.
          The operation is guaranteed to succeed at undoing all resource
          changes, after which point it terminates with a ``CANCELLED``
          status.
        - All other attempts to modify the instance are rejected.
        - Reading the instance via the API continues to give the
          pre-request resource levels.

        Upon completion of the returned operation:

        - Billing begins for all successfully-allocated resources (some
          types may have lower than the requested levels).
        - All newly-reserved resources are available for serving the
          instance's tables.
        - The instance's new resource levels are readable via the API.

        The returned long-running operation will have a name of the
        format ``<instance_name>/operations/<operation_id>`` and can be
        used to track the instance modification. The metadata field type
        is
        [UpdateInstanceMetadata][google.spanner.admin.instance.v1.UpdateInstanceMetadata].
        The response field type is
        [Instance][google.spanner.admin.instance.v1.Instance], if
        successful.

        Authorization requires ``spanner.instances.update`` permission
        on the resource
        [name][google.spanner.admin.instance.v1.Instance.name].

        Returns:
            Callable[[~.UpdateInstanceRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_instance" not in self._stubs:
            self._stubs["update_instance"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/UpdateInstance",
                request_serializer=spanner_instance_admin.UpdateInstanceRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["update_instance"]

    @property
    def delete_instance(
        self,
    ) -> Callable[
        [spanner_instance_admin.DeleteInstanceRequest], Awaitable[empty_pb2.Empty]
    ]:
        r"""Return a callable for the delete instance method over gRPC.

        Deletes an instance.

        Immediately upon completion of the request:

        - Billing ceases for all of the instance's reserved resources.

        Soon afterward:

        - The instance and *all of its databases* immediately and
          irrevocably disappear from the API. All data in the databases
          is permanently deleted.

        Returns:
            Callable[[~.DeleteInstanceRequest],
                    Awaitable[~.Empty]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_instance" not in self._stubs:
            self._stubs["delete_instance"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/DeleteInstance",
                request_serializer=spanner_instance_admin.DeleteInstanceRequest.serialize,
                response_deserializer=empty_pb2.Empty.FromString,
            )
        return self._stubs["delete_instance"]

    @property
    def set_iam_policy(
        self,
    ) -> Callable[[iam_policy_pb2.SetIamPolicyRequest], Awaitable[policy_pb2.Policy]]:
        r"""Return a callable for the set iam policy method over gRPC.

        Sets the access control policy on an instance resource. Replaces
        any existing policy.

        Authorization requires ``spanner.instances.setIamPolicy`` on
        [resource][google.iam.v1.SetIamPolicyRequest.resource].

        Returns:
            Callable[[~.SetIamPolicyRequest],
                    Awaitable[~.Policy]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "set_iam_policy" not in self._stubs:
            self._stubs["set_iam_policy"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/SetIamPolicy",
                request_serializer=iam_policy_pb2.SetIamPolicyRequest.SerializeToString,
                response_deserializer=policy_pb2.Policy.FromString,
            )
        return self._stubs["set_iam_policy"]

    @property
    def get_iam_policy(
        self,
    ) -> Callable[[iam_policy_pb2.GetIamPolicyRequest], Awaitable[policy_pb2.Policy]]:
        r"""Return a callable for the get iam policy method over gRPC.

        Gets the access control policy for an instance resource. Returns
        an empty policy if an instance exists but does not have a policy
        set.

        Authorization requires ``spanner.instances.getIamPolicy`` on
        [resource][google.iam.v1.GetIamPolicyRequest.resource].

        Returns:
            Callable[[~.GetIamPolicyRequest],
                    Awaitable[~.Policy]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_iam_policy" not in self._stubs:
            self._stubs["get_iam_policy"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/GetIamPolicy",
                request_serializer=iam_policy_pb2.GetIamPolicyRequest.SerializeToString,
                response_deserializer=policy_pb2.Policy.FromString,
            )
        return self._stubs["get_iam_policy"]

    @property
    def test_iam_permissions(
        self,
    ) -> Callable[
        [iam_policy_pb2.TestIamPermissionsRequest],
        Awaitable[iam_policy_pb2.TestIamPermissionsResponse],
    ]:
        r"""Return a callable for the test iam permissions method over gRPC.

        Returns permissions that the caller has on the specified
        instance resource.

        Attempting this RPC on a non-existent Cloud Spanner instance
        resource will result in a NOT_FOUND error if the user has
        ``spanner.instances.list`` permission on the containing Google
        Cloud Project. Otherwise returns an empty set of permissions.

        Returns:
            Callable[[~.TestIamPermissionsRequest],
                    Awaitable[~.TestIamPermissionsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "test_iam_permissions" not in self._stubs:
            self._stubs["test_iam_permissions"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/TestIamPermissions",
                request_serializer=iam_policy_pb2.TestIamPermissionsRequest.SerializeToString,
                response_deserializer=iam_policy_pb2.TestIamPermissionsResponse.FromString,
            )
        return self._stubs["test_iam_permissions"]

    @property
    def get_instance_partition(
        self,
    ) -> Callable[
        [spanner_instance_admin.GetInstancePartitionRequest],
        Awaitable[spanner_instance_admin.InstancePartition],
    ]:
        r"""Return a callable for the get instance partition method over gRPC.

        Gets information about a particular instance
        partition.

        Returns:
            Callable[[~.GetInstancePartitionRequest],
                    Awaitable[~.InstancePartition]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_instance_partition" not in self._stubs:
            self._stubs["get_instance_partition"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/GetInstancePartition",
                request_serializer=spanner_instance_admin.GetInstancePartitionRequest.serialize,
                response_deserializer=spanner_instance_admin.InstancePartition.deserialize,
            )
        return self._stubs["get_instance_partition"]

    @property
    def create_instance_partition(
        self,
    ) -> Callable[
        [spanner_instance_admin.CreateInstancePartitionRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the create instance partition method over gRPC.

        Creates an instance partition and begins preparing it to be
        used. The returned long-running operation can be used to track
        the progress of preparing the new instance partition. The
        instance partition name is assigned by the caller. If the named
        instance partition already exists, ``CreateInstancePartition``
        returns ``ALREADY_EXISTS``.

        Immediately upon completion of this request:

        - The instance partition is readable via the API, with all
          requested attributes but no allocated resources. Its state is
          ``CREATING``.

        Until completion of the returned operation:

        - Cancelling the operation renders the instance partition
          immediately unreadable via the API.
        - The instance partition can be deleted.
        - All other attempts to modify the instance partition are
          rejected.

        Upon completion of the returned operation:

        - Billing for all successfully-allocated resources begins (some
          types may have lower than the requested levels).
        - Databases can start using this instance partition.
        - The instance partition's allocated resource levels are
          readable via the API.
        - The instance partition's state becomes ``READY``.

        The returned long-running operation will have a name of the
        format ``<instance_partition_name>/operations/<operation_id>``
        and can be used to track creation of the instance partition. The
        metadata field type is
        [CreateInstancePartitionMetadata][google.spanner.admin.instance.v1.CreateInstancePartitionMetadata].
        The response field type is
        [InstancePartition][google.spanner.admin.instance.v1.InstancePartition],
        if successful.

        Returns:
            Callable[[~.CreateInstancePartitionRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_instance_partition" not in self._stubs:
            self._stubs["create_instance_partition"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/CreateInstancePartition",
                request_serializer=spanner_instance_admin.CreateInstancePartitionRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_instance_partition"]

    @property
    def delete_instance_partition(
        self,
    ) -> Callable[
        [spanner_instance_admin.DeleteInstancePartitionRequest],
        Awaitable[empty_pb2.Empty],
    ]:
        r"""Return a callable for the delete instance partition method over gRPC.

        Deletes an existing instance partition. Requires that the
        instance partition is not used by any database or backup and is
        not the default instance partition of an instance.

        Authorization requires ``spanner.instancePartitions.delete``
        permission on the resource
        [name][google.spanner.admin.instance.v1.InstancePartition.name].

        Returns:
            Callable[[~.DeleteInstancePartitionRequest],
                    Awaitable[~.Empty]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_instance_partition" not in self._stubs:
            self._stubs["delete_instance_partition"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/DeleteInstancePartition",
                request_serializer=spanner_instance_admin.DeleteInstancePartitionRequest.serialize,
                response_deserializer=empty_pb2.Empty.FromString,
            )
        return self._stubs["delete_instance_partition"]

    @property
    def update_instance_partition(
        self,
    ) -> Callable[
        [spanner_instance_admin.UpdateInstancePartitionRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the update instance partition method over gRPC.

        Updates an instance partition, and begins allocating or
        releasing resources as requested. The returned long-running
        operation can be used to track the progress of updating the
        instance partition. If the named instance partition does not
        exist, returns ``NOT_FOUND``.

        Immediately upon completion of this request:

        - For resource types for which a decrease in the instance
          partition's allocation has been requested, billing is based on
          the newly-requested level.

        Until completion of the returned operation:

        - Cancelling the operation sets its metadata's
          [cancel_time][google.spanner.admin.instance.v1.UpdateInstancePartitionMetadata.cancel_time],
          and begins restoring resources to their pre-request values.
          The operation is guaranteed to succeed at undoing all resource
          changes, after which point it terminates with a ``CANCELLED``
          status.
        - All other attempts to modify the instance partition are
          rejected.
        - Reading the instance partition via the API continues to give
          the pre-request resource levels.

        Upon completion of the returned operation:

        - Billing begins for all successfully-allocated resources (some
          types may have lower than the requested levels).
        - All newly-reserved resources are available for serving the
          instance partition's tables.
        - The instance partition's new resource levels are readable via
          the API.

        The returned long-running operation will have a name of the
        format ``<instance_partition_name>/operations/<operation_id>``
        and can be used to track the instance partition modification.
        The metadata field type is
        [UpdateInstancePartitionMetadata][google.spanner.admin.instance.v1.UpdateInstancePartitionMetadata].
        The response field type is
        [InstancePartition][google.spanner.admin.instance.v1.InstancePartition],
        if successful.

        Authorization requires ``spanner.instancePartitions.update``
        permission on the resource
        [name][google.spanner.admin.instance.v1.InstancePartition.name].

        Returns:
            Callable[[~.UpdateInstancePartitionRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_instance_partition" not in self._stubs:
            self._stubs["update_instance_partition"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/UpdateInstancePartition",
                request_serializer=spanner_instance_admin.UpdateInstancePartitionRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["update_instance_partition"]

    @property
    def list_instance_partition_operations(
        self,
    ) -> Callable[
        [spanner_instance_admin.ListInstancePartitionOperationsRequest],
        Awaitable[spanner_instance_admin.ListInstancePartitionOperationsResponse],
    ]:
        r"""Return a callable for the list instance partition
        operations method over gRPC.

        Lists instance partition long-running operations in the given
        instance. An instance partition operation has a name of the form
        ``projects/<project>/instances/<instance>/instancePartitions/<instance_partition>/operations/<operation>``.
        The long-running operation metadata field type
        ``metadata.type_url`` describes the type of the metadata.
        Operations returned include those that have
        completed/failed/canceled within the last 7 days, and pending
        operations. Operations returned are ordered by
        ``operation.metadata.value.start_time`` in descending order
        starting from the most recently started operation.

        Authorization requires
        ``spanner.instancePartitionOperations.list`` permission on the
        resource
        [parent][google.spanner.admin.instance.v1.ListInstancePartitionOperationsRequest.parent].

        Returns:
            Callable[[~.ListInstancePartitionOperationsRequest],
                    Awaitable[~.ListInstancePartitionOperationsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_instance_partition_operations" not in self._stubs:
            self._stubs[
                "list_instance_partition_operations"
            ] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/ListInstancePartitionOperations",
                request_serializer=spanner_instance_admin.ListInstancePartitionOperationsRequest.serialize,
                response_deserializer=spanner_instance_admin.ListInstancePartitionOperationsResponse.deserialize,
            )
        return self._stubs["list_instance_partition_operations"]

    @property
    def move_instance(
        self,
    ) -> Callable[
        [spanner_instance_admin.MoveInstanceRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the move instance method over gRPC.

        Moves an instance to the target instance configuration. You can
        use the returned long-running operation to track the progress of
        moving the instance.

        ``MoveInstance`` returns ``FAILED_PRECONDITION`` if the instance
        meets any of the following criteria:

        - Is undergoing a move to a different instance configuration
        - Has backups
        - Has an ongoing update
        - Contains any CMEK-enabled databases
        - Is a free trial instance

        While the operation is pending:

        - All other attempts to modify the instance, including changes
          to its compute capacity, are rejected.

        - The following database and backup admin operations are
          rejected:

          - ``DatabaseAdmin.CreateDatabase``
          - ``DatabaseAdmin.UpdateDatabaseDdl`` (disabled if
            default_leader is specified in the request.)
          - ``DatabaseAdmin.RestoreDatabase``
          - ``DatabaseAdmin.CreateBackup``
          - ``DatabaseAdmin.CopyBackup``

        - Both the source and target instance configurations are subject
          to hourly compute and storage charges.

        - The instance might experience higher read-write latencies and
          a higher transaction abort rate. However, moving an instance
          doesn't cause any downtime.

        The returned long-running operation has a name of the format
        ``<instance_name>/operations/<operation_id>`` and can be used to
        track the move instance operation. The metadata field type is
        [MoveInstanceMetadata][google.spanner.admin.instance.v1.MoveInstanceMetadata].
        The response field type is
        [Instance][google.spanner.admin.instance.v1.Instance], if
        successful. Cancelling the operation sets its metadata's
        [cancel_time][google.spanner.admin.instance.v1.MoveInstanceMetadata.cancel_time].
        Cancellation is not immediate because it involves moving any
        data previously moved to the target instance configuration back
        to the original instance configuration. You can use this
        operation to track the progress of the cancellation. Upon
        successful completion of the cancellation, the operation
        terminates with ``CANCELLED`` status.

        If not cancelled, upon completion of the returned operation:

        - The instance successfully moves to the target instance
          configuration.
        - You are billed for compute and storage in target instance
          configuration.

        Authorization requires the ``spanner.instances.update``
        permission on the resource
        [instance][google.spanner.admin.instance.v1.Instance].

        For more details, see `Move an
        instance <https://cloud.google.com/spanner/docs/move-instance>`__.

        Returns:
            Callable[[~.MoveInstanceRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "move_instance" not in self._stubs:
            self._stubs["move_instance"] = self._logged_channel.unary_unary(
                "/google.spanner.admin.instance.v1.InstanceAdmin/MoveInstance",
                request_serializer=spanner_instance_admin.MoveInstanceRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["move_instance"]

    def _prep_wrapped_messages(self, client_info):
        """Precompute the wrapped methods, overriding the base class method to use async wrappers."""
        self._wrapped_methods = {
            self.list_instance_configs: self._wrap_method(
                self.list_instance_configs,
                default_retry=retries.AsyncRetry(
                    initial=1.0,
                    maximum=32.0,
                    multiplier=1.3,
                    predicate=retries.if_exception_type(
                        core_exceptions.DeadlineExceeded,
                        core_exceptions.ServiceUnavailable,
                    ),
                    deadline=3600.0,
                ),
                default_timeout=3600.0,
                client_info=client_info,
            ),
            self.get_instance_config: self._wrap_method(
                self.get_instance_config,
                default_retry=retries.AsyncRetry(
                    initial=1.0,
                    maximum=32.0,
                    multiplier=1.3,
                    predicate=retries.if_exception_type(
                        core_exceptions.DeadlineExceeded,
                        core_exceptions.ServiceUnavailable,
                    ),
                    deadline=3600.0,
                ),
                default_timeout=3600.0,
                client_info=client_info,
            ),
            self.create_instance_config: self._wrap_method(
                self.create_instance_config,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_instance_config: self._wrap_method(
                self.update_instance_config,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_instance_config: self._wrap_method(
                self.delete_instance_config,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_instance_config_operations: self._wrap_method(
                self.list_instance_config_operations,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_instances: self._wrap_method(
                self.list_instances,
                default_retry=retries.AsyncRetry(
                    initial=1.0,
                    maximum=32.0,
                    multiplier=1.3,
                    predicate=retries.if_exception_type(
                        core_exceptions.DeadlineExceeded,
                        core_exceptions.ServiceUnavailable,
                    ),
                    deadline=3600.0,
                ),
                default_timeout=3600.0,
                client_info=client_info,
            ),
            self.list_instance_partitions: self._wrap_method(
                self.list_instance_partitions,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_instance: self._wrap_method(
                self.get_instance,
                default_retry=retries.AsyncRetry(
                    initial=1.0,
                    maximum=32.0,
                    multiplier=1.3,
                    predicate=retries.if_exception_type(
                        core_exceptions.DeadlineExceeded,
                        core_exceptions.ServiceUnavailable,
                    ),
                    deadline=3600.0,
                ),
                default_timeout=3600.0,
                client_info=client_info,
            ),
            self.create_instance: self._wrap_method(
                self.create_instance,
                default_timeout=3600.0,
                client_info=client_info,
            ),
            self.update_instance: self._wrap_method(
                self.update_instance,
                default_timeout=3600.0,
                client_info=client_info,
            ),
            self.delete_instance: self._wrap_method(
                self.delete_instance,
                default_retry=retries.AsyncRetry(
                    initial=1.0,
                    maximum=32.0,
                    multiplier=1.3,
                    predicate=retries.if_exception_type(
                        core_exceptions.DeadlineExceeded,
                        core_exceptions.ServiceUnavailable,
                    ),
                    deadline=3600.0,
                ),
                default_timeout=3600.0,
                client_info=client_info,
            ),
            self.set_iam_policy: self._wrap_method(
                self.set_iam_policy,
                default_timeout=30.0,
                client_info=client_info,
            ),
            self.get_iam_policy: self._wrap_method(
                self.get_iam_policy,
                default_retry=retries.AsyncRetry(
                    initial=1.0,
                    maximum=32.0,
                    multiplier=1.3,
                    predicate=retries.if_exception_type(
                        core_exceptions.DeadlineExceeded,
                        core_exceptions.ServiceUnavailable,
                    ),
                    deadline=30.0,
                ),
                default_timeout=30.0,
                client_info=client_info,
            ),
            self.test_iam_permissions: self._wrap_method(
                self.test_iam_permissions,
                default_timeout=30.0,
                client_info=client_info,
            ),
            self.get_instance_partition: self._wrap_method(
                self.get_instance_partition,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_instance_partition: self._wrap_method(
                self.create_instance_partition,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_instance_partition: self._wrap_method(
                self.delete_instance_partition,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_instance_partition: self._wrap_method(
                self.update_instance_partition,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_instance_partition_operations: self._wrap_method(
                self.list_instance_partition_operations,
                default_timeout=None,
                client_info=client_info,
            ),
            self.move_instance: self._wrap_method(
                self.move_instance,
                default_timeout=None,
                client_info=client_info,
            ),
            self.cancel_operation: self._wrap_method(
                self.cancel_operation,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_operation: self._wrap_method(
                self.delete_operation,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_operation: self._wrap_method(
                self.get_operation,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_operations: self._wrap_method(
                self.list_operations,
                default_timeout=None,
                client_info=client_info,
            ),
        }

    def _wrap_method(self, func, *args, **kwargs):
        if self._wrap_with_kind:  # pragma: NO COVER
            kwargs["kind"] = self.kind
        return gapic_v1.method_async.wrap_method(func, *args, **kwargs)

    def close(self):
        return self._logged_channel.close()

    @property
    def kind(self) -> str:
        return "grpc_asyncio"

    @property
    def delete_operation(
        self,
    ) -> Callable[[operations_pb2.DeleteOperationRequest], None]:
        r"""Return a callable for the delete_operation method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_operation" not in self._stubs:
            self._stubs["delete_operation"] = self._logged_channel.unary_unary(
                "/google.longrunning.Operations/DeleteOperation",
                request_serializer=operations_pb2.DeleteOperationRequest.SerializeToString,
                response_deserializer=None,
            )
        return self._stubs["delete_operation"]

    @property
    def cancel_operation(
        self,
    ) -> Callable[[operations_pb2.CancelOperationRequest], None]:
        r"""Return a callable for the cancel_operation method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "cancel_operation" not in self._stubs:
            self._stubs["cancel_operation"] = self._logged_channel.unary_unary(
                "/google.longrunning.Operations/CancelOperation",
                request_serializer=operations_pb2.CancelOperationRequest.SerializeToString,
                response_deserializer=None,
            )
        return self._stubs["cancel_operation"]

    @property
    def get_operation(
        self,
    ) -> Callable[[operations_pb2.GetOperationRequest], operations_pb2.Operation]:
        r"""Return a callable for the get_operation method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_operation" not in self._stubs:
            self._stubs["get_operation"] = self._logged_channel.unary_unary(
                "/google.longrunning.Operations/GetOperation",
                request_serializer=operations_pb2.GetOperationRequest.SerializeToString,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["get_operation"]

    @property
    def list_operations(
        self,
    ) -> Callable[
        [operations_pb2.ListOperationsRequest], operations_pb2.ListOperationsResponse
    ]:
        r"""Return a callable for the list_operations method over gRPC."""
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_operations" not in self._stubs:
            self._stubs["list_operations"] = self._logged_channel.unary_unary(
                "/google.longrunning.Operations/ListOperations",
                request_serializer=operations_pb2.ListOperationsRequest.SerializeToString,
                response_deserializer=operations_pb2.ListOperationsResponse.FromString,
            )
        return self._stubs["list_operations"]


__all__ = ("InstanceAdminGrpcAsyncIOTransport",)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.bigtable_admin_v2.types import common
from google.cloud.bigtable_admin_v2.types import table as gba_table
from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.bigtable.admin.v2",
    manifest={
        "RestoreTableRequest",
        "RestoreTableMetadata",
        "OptimizeRestoredTableMetadata",
        "CreateTableRequest",
        "CreateTableFromSnapshotRequest",
        "DropRowRangeRequest",
        "ListTablesRequest",
        "ListTablesResponse",
        "GetTableRequest",
        "UpdateTableRequest",
        "UpdateTableMetadata",
        "DeleteTableRequest",
        "UndeleteTableRequest",
        "UndeleteTableMetadata",
        "ModifyColumnFamiliesRequest",
        "GenerateConsistencyTokenRequest",
        "GenerateConsistencyTokenResponse",
        "CheckConsistencyRequest",
        "StandardReadRemoteWrites",
        "DataBoostReadLocalWrites",
        "CheckConsistencyResponse",
        "SnapshotTableRequest",
        "GetSnapshotRequest",
        "ListSnapshotsRequest",
        "ListSnapshotsResponse",
        "DeleteSnapshotRequest",
        "SnapshotTableMetadata",
        "CreateTableFromSnapshotMetadata",
        "CreateBackupRequest",
        "CreateBackupMetadata",
        "UpdateBackupRequest",
        "GetBackupRequest",
        "DeleteBackupRequest",
        "ListBackupsRequest",
        "ListBackupsResponse",
        "CopyBackupRequest",
        "CopyBackupMetadata",
        "CreateAuthorizedViewRequest",
        "CreateAuthorizedViewMetadata",
        "ListAuthorizedViewsRequest",
        "ListAuthorizedViewsResponse",
        "GetAuthorizedViewRequest",
        "UpdateAuthorizedViewRequest",
        "UpdateAuthorizedViewMetadata",
        "DeleteAuthorizedViewRequest",
        "CreateSchemaBundleRequest",
        "CreateSchemaBundleMetadata",
        "UpdateSchemaBundleRequest",
        "UpdateSchemaBundleMetadata",
        "GetSchemaBundleRequest",
        "ListSchemaBundlesRequest",
        "ListSchemaBundlesResponse",
        "DeleteSchemaBundleRequest",
    },
)


class RestoreTableRequest(proto.Message):
    r"""The request for
    [RestoreTable][google.bigtable.admin.v2.BigtableTableAdmin.RestoreTable].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        parent (str):
            Required. The name of the instance in which to create the
            restored table. Values are of the form
            ``projects/<project>/instances/<instance>``.
        table_id (str):
            Required. The id of the table to create and restore to. This
            table must not already exist. The ``table_id`` appended to
            ``parent`` forms the full table name of the form
            ``projects/<project>/instances/<instance>/tables/<table_id>``.
        backup (str):
            Name of the backup from which to restore. Values are of the
            form
            ``projects/<project>/instances/<instance>/clusters/<cluster>/backups/<backup>``.

            This field is a member of `oneof`_ ``source``.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    table_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    backup: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="source",
    )


class RestoreTableMetadata(proto.Message):
    r"""Metadata type for the long-running operation returned by
    [RestoreTable][google.bigtable.admin.v2.BigtableTableAdmin.RestoreTable].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            Name of the table being created and restored
            to.
        source_type (google.cloud.bigtable_admin_v2.types.RestoreSourceType):
            The type of the restore source.
        backup_info (google.cloud.bigtable_admin_v2.types.BackupInfo):

            This field is a member of `oneof`_ ``source_info``.
        optimize_table_operation_name (str):
            If exists, the name of the long-running operation that will
            be used to track the post-restore optimization process to
            optimize the performance of the restored table. The metadata
            type of the long-running operation is
            [OptimizeRestoreTableMetadata][]. The response type is
            [Empty][google.protobuf.Empty]. This long-running operation
            may be automatically created by the system if applicable
            after the RestoreTable long-running operation completes
            successfully. This operation may not be created if the table
            is already optimized or the restore was not successful.
        progress (google.cloud.bigtable_admin_v2.types.OperationProgress):
            The progress of the
            [RestoreTable][google.bigtable.admin.v2.BigtableTableAdmin.RestoreTable]
            operation.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    source_type: gba_table.RestoreSourceType = proto.Field(
        proto.ENUM,
        number=2,
        enum=gba_table.RestoreSourceType,
    )
    backup_info: gba_table.BackupInfo = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="source_info",
        message=gba_table.BackupInfo,
    )
    optimize_table_operation_name: str = proto.Field(
        proto.STRING,
        number=4,
    )
    progress: common.OperationProgress = proto.Field(
        proto.MESSAGE,
        number=5,
        message=common.OperationProgress,
    )


class OptimizeRestoredTableMetadata(proto.Message):
    r"""Metadata type for the long-running operation used to track
    the progress of optimizations performed on a newly restored
    table. This long-running operation is automatically created by
    the system after the successful completion of a table restore,
    and cannot be cancelled.

    Attributes:
        name (str):
            Name of the restored table being optimized.
        progress (google.cloud.bigtable_admin_v2.types.OperationProgress):
            The progress of the post-restore
            optimizations.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    progress: common.OperationProgress = proto.Field(
        proto.MESSAGE,
        number=2,
        message=common.OperationProgress,
    )


class CreateTableRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.CreateTable][google.bigtable.admin.v2.BigtableTableAdmin.CreateTable]

    Attributes:
        parent (str):
            Required. The unique name of the instance in which to create
            the table. Values are of the form
            ``projects/{project}/instances/{instance}``.
        table_id (str):
            Required. The name by which the new table should be referred
            to within the parent instance, e.g., ``foobar`` rather than
            ``{parent}/tables/foobar``. Maximum 50 characters.
        table (google.cloud.bigtable_admin_v2.types.Table):
            Required. The Table to create.
        initial_splits (MutableSequence[google.cloud.bigtable_admin_v2.types.CreateTableRequest.Split]):
            The optional list of row keys that will be used to initially
            split the table into several tablets (tablets are similar to
            HBase regions). Given two split keys, ``s1`` and ``s2``,
            three tablets will be created, spanning the key ranges:
            ``[, s1), [s1, s2), [s2, )``.

            Example:

            - Row keys :=
              ``["a", "apple", "custom", "customer_1", "customer_2",``
              ``"other", "zz"]``
            - initial_split_keys :=
              ``["apple", "customer_1", "customer_2", "other"]``
            - Key assignment:

              - Tablet 1 ``[, apple)                => {"a"}.``
              - Tablet 2
                ``[apple, customer_1)      => {"apple", "custom"}.``
              - Tablet 3 ``[customer_1, customer_2) => {"customer_1"}.``
              - Tablet 4 ``[customer_2, other)      => {"customer_2"}.``
              - Tablet 5
                ``[other, )                => {"other", "zz"}.``
    """

    class Split(proto.Message):
        r"""An initial split point for a newly created table.

        Attributes:
            key (bytes):
                Row key to use as an initial tablet boundary.
        """

        key: bytes = proto.Field(
            proto.BYTES,
            number=1,
        )

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    table_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    table: gba_table.Table = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gba_table.Table,
    )
    initial_splits: MutableSequence[Split] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=Split,
    )


class CreateTableFromSnapshotRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.CreateTableFromSnapshot][google.bigtable.admin.v2.BigtableTableAdmin.CreateTableFromSnapshot]

    Note: This is a private alpha release of Cloud Bigtable snapshots.
    This feature is not currently available to most Cloud Bigtable
    customers. This feature might be changed in backward-incompatible
    ways and is not recommended for production use. It is not subject to
    any SLA or deprecation policy.

    Attributes:
        parent (str):
            Required. The unique name of the instance in which to create
            the table. Values are of the form
            ``projects/{project}/instances/{instance}``.
        table_id (str):
            Required. The name by which the new table should be referred
            to within the parent instance, e.g., ``foobar`` rather than
            ``{parent}/tables/foobar``.
        source_snapshot (str):
            Required. The unique name of the snapshot from which to
            restore the table. The snapshot and the table must be in the
            same instance. Values are of the form
            ``projects/{project}/instances/{instance}/clusters/{cluster}/snapshots/{snapshot}``.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    table_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    source_snapshot: str = proto.Field(
        proto.STRING,
        number=3,
    )


class DropRowRangeRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.DropRowRange][google.bigtable.admin.v2.BigtableTableAdmin.DropRowRange]

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            Required. The unique name of the table on which to drop a
            range of rows. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
        row_key_prefix (bytes):
            Delete all rows that start with this row key
            prefix. Prefix cannot be zero length.

            This field is a member of `oneof`_ ``target``.
        delete_all_data_from_table (bool):
            Delete all rows in the table. Setting this to
            false is a no-op.

            This field is a member of `oneof`_ ``target``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    row_key_prefix: bytes = proto.Field(
        proto.BYTES,
        number=2,
        oneof="target",
    )
    delete_all_data_from_table: bool = proto.Field(
        proto.BOOL,
        number=3,
        oneof="target",
    )


class ListTablesRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.ListTables][google.bigtable.admin.v2.BigtableTableAdmin.ListTables]

    Attributes:
        parent (str):
            Required. The unique name of the instance for which tables
            should be listed. Values are of the form
            ``projects/{project}/instances/{instance}``.
        view (google.cloud.bigtable_admin_v2.types.Table.View):
            The view to be applied to the returned tables' fields.
            NAME_ONLY view (default) and REPLICATION_VIEW are supported.
        page_size (int):
            Maximum number of results per page.

            A page_size of zero lets the server choose the number of
            items to return. A page_size which is strictly positive will
            return at most that many items. A negative page_size will
            cause an error.

            Following the first request, subsequent paginated calls are
            not required to pass a page_size. If a page_size is set in
            subsequent calls, it must match the page_size given in the
            first request.
        page_token (str):
            The value of ``next_page_token`` returned by a previous
            call.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    view: gba_table.Table.View = proto.Field(
        proto.ENUM,
        number=2,
        enum=gba_table.Table.View,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=4,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListTablesResponse(proto.Message):
    r"""Response message for
    [google.bigtable.admin.v2.BigtableTableAdmin.ListTables][google.bigtable.admin.v2.BigtableTableAdmin.ListTables]

    Attributes:
        tables (MutableSequence[google.cloud.bigtable_admin_v2.types.Table]):
            The tables present in the requested instance.
        next_page_token (str):
            Set if not all tables could be returned in a single
            response. Pass this value to ``page_token`` in another
            request to get the next page of results.
    """

    @property
    def raw_page(self):
        return self

    tables: MutableSequence[gba_table.Table] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_table.Table,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GetTableRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.GetTable][google.bigtable.admin.v2.BigtableTableAdmin.GetTable]

    Attributes:
        name (str):
            Required. The unique name of the requested table. Values are
            of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
        view (google.cloud.bigtable_admin_v2.types.Table.View):
            The view to be applied to the returned table's fields.
            Defaults to ``SCHEMA_VIEW`` if unspecified.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    view: gba_table.Table.View = proto.Field(
        proto.ENUM,
        number=2,
        enum=gba_table.Table.View,
    )


class UpdateTableRequest(proto.Message):
    r"""The request for
    [UpdateTable][google.bigtable.admin.v2.BigtableTableAdmin.UpdateTable].

    Attributes:
        table (google.cloud.bigtable_admin_v2.types.Table):
            Required. The table to update. The table's ``name`` field is
            used to identify the table to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The list of fields to update. A mask specifying
            which fields (e.g. ``change_stream_config``) in the
            ``table`` field should be updated. This mask is relative to
            the ``table`` field, not to the request message. The
            wildcard (\*) path is currently not supported. Currently
            UpdateTable is only supported for the following fields:

            - ``change_stream_config``
            - ``change_stream_config.retention_period``
            - ``deletion_protection``
            - ``row_key_schema``

            If ``column_families`` is set in ``update_mask``, it will
            return an UNIMPLEMENTED error.
        ignore_warnings (bool):
            Optional. If true, ignore safety checks when
            updating the table.
    """

    table: gba_table.Table = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gba_table.Table,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    ignore_warnings: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class UpdateTableMetadata(proto.Message):
    r"""Metadata type for the operation returned by
    [UpdateTable][google.bigtable.admin.v2.BigtableTableAdmin.UpdateTable].

    Attributes:
        name (str):
            The name of the table being updated.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which this operation started.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            If set, the time at which this operation
            finished or was canceled.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class DeleteTableRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.DeleteTable][google.bigtable.admin.v2.BigtableTableAdmin.DeleteTable]

    Attributes:
        name (str):
            Required. The unique name of the table to be deleted. Values
            are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class UndeleteTableRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.UndeleteTable][google.bigtable.admin.v2.BigtableTableAdmin.UndeleteTable]

    Attributes:
        name (str):
            Required. The unique name of the table to be restored.
            Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class UndeleteTableMetadata(proto.Message):
    r"""Metadata type for the operation returned by
    [google.bigtable.admin.v2.BigtableTableAdmin.UndeleteTable][google.bigtable.admin.v2.BigtableTableAdmin.UndeleteTable].

    Attributes:
        name (str):
            The name of the table being restored.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which this operation started.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            If set, the time at which this operation
            finished or was cancelled.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class ModifyColumnFamiliesRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.ModifyColumnFamilies][google.bigtable.admin.v2.BigtableTableAdmin.ModifyColumnFamilies]

    Attributes:
        name (str):
            Required. The unique name of the table whose families should
            be modified. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
        modifications (MutableSequence[google.cloud.bigtable_admin_v2.types.ModifyColumnFamiliesRequest.Modification]):
            Required. Modifications to be atomically
            applied to the specified table's families.
            Entries are applied in order, meaning that
            earlier modifications can be masked by later
            ones (in the case of repeated updates to the
            same family, for example).
        ignore_warnings (bool):
            Optional. If true, ignore safety checks when
            modifying the column families.
    """

    class Modification(proto.Message):
        r"""A create, update, or delete of a particular column family.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            id (str):
                The ID of the column family to be modified.
            create (google.cloud.bigtable_admin_v2.types.ColumnFamily):
                Create a new column family with the specified
                schema, or fail if one already exists with the
                given ID.

                This field is a member of `oneof`_ ``mod``.
            update (google.cloud.bigtable_admin_v2.types.ColumnFamily):
                Update an existing column family to the
                specified schema, or fail if no column family
                exists with the given ID.

                This field is a member of `oneof`_ ``mod``.
            drop (bool):
                Drop (delete) the column family with the
                given ID, or fail if no such family exists.

                This field is a member of `oneof`_ ``mod``.
            update_mask (google.protobuf.field_mask_pb2.FieldMask):
                Optional. A mask specifying which fields (e.g. ``gc_rule``)
                in the ``update`` mod should be updated, ignored for other
                modification types. If unset or empty, we treat it as
                updating ``gc_rule`` to be backward compatible.
        """

        id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        create: gba_table.ColumnFamily = proto.Field(
            proto.MESSAGE,
            number=2,
            oneof="mod",
            message=gba_table.ColumnFamily,
        )
        update: gba_table.ColumnFamily = proto.Field(
            proto.MESSAGE,
            number=3,
            oneof="mod",
            message=gba_table.ColumnFamily,
        )
        drop: bool = proto.Field(
            proto.BOOL,
            number=4,
            oneof="mod",
        )
        update_mask: field_mask_pb2.FieldMask = proto.Field(
            proto.MESSAGE,
            number=6,
            message=field_mask_pb2.FieldMask,
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    modifications: MutableSequence[Modification] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=Modification,
    )
    ignore_warnings: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class GenerateConsistencyTokenRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.GenerateConsistencyToken][google.bigtable.admin.v2.BigtableTableAdmin.GenerateConsistencyToken]

    Attributes:
        name (str):
            Required. The unique name of the Table for which to create a
            consistency token. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class GenerateConsistencyTokenResponse(proto.Message):
    r"""Response message for
    [google.bigtable.admin.v2.BigtableTableAdmin.GenerateConsistencyToken][google.bigtable.admin.v2.BigtableTableAdmin.GenerateConsistencyToken]

    Attributes:
        consistency_token (str):
            The generated consistency token.
    """

    consistency_token: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CheckConsistencyRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.CheckConsistency][google.bigtable.admin.v2.BigtableTableAdmin.CheckConsistency]

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            Required. The unique name of the Table for which to check
            replication consistency. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
        consistency_token (str):
            Required. The token created using
            GenerateConsistencyToken for the Table.
        standard_read_remote_writes (google.cloud.bigtable_admin_v2.types.StandardReadRemoteWrites):
            Checks that reads using an app profile with
            ``StandardIsolation`` can see all writes committed before
            the token was created, even if the read and write target
            different clusters.

            This field is a member of `oneof`_ ``mode``.
        data_boost_read_local_writes (google.cloud.bigtable_admin_v2.types.DataBoostReadLocalWrites):
            Checks that reads using an app profile with
            ``DataBoostIsolationReadOnly`` can see all writes committed
            before the token was created, but only if the read and write
            target the same cluster.

            This field is a member of `oneof`_ ``mode``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    consistency_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    standard_read_remote_writes: "StandardReadRemoteWrites" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="mode",
        message="StandardReadRemoteWrites",
    )
    data_boost_read_local_writes: "DataBoostReadLocalWrites" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="mode",
        message="DataBoostReadLocalWrites",
    )


class StandardReadRemoteWrites(proto.Message):
    r"""Checks that all writes before the consistency token was
    generated are replicated in every cluster and readable.

    """


class DataBoostReadLocalWrites(proto.Message):
    r"""Checks that all writes before the consistency token was
    generated in the same cluster are readable by Databoost.

    """


class CheckConsistencyResponse(proto.Message):
    r"""Response message for
    [google.bigtable.admin.v2.BigtableTableAdmin.CheckConsistency][google.bigtable.admin.v2.BigtableTableAdmin.CheckConsistency]

    Attributes:
        consistent (bool):
            True only if the token is consistent. A token
            is consistent if replication has caught up with
            the restrictions specified in the request.
    """

    consistent: bool = proto.Field(
        proto.BOOL,
        number=1,
    )


class SnapshotTableRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.SnapshotTable][google.bigtable.admin.v2.BigtableTableAdmin.SnapshotTable]

    Note: This is a private alpha release of Cloud Bigtable snapshots.
    This feature is not currently available to most Cloud Bigtable
    customers. This feature might be changed in backward-incompatible
    ways and is not recommended for production use. It is not subject to
    any SLA or deprecation policy.

    Attributes:
        name (str):
            Required. The unique name of the table to have the snapshot
            taken. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
        cluster (str):
            Required. The name of the cluster where the snapshot will be
            created in. Values are of the form
            ``projects/{project}/instances/{instance}/clusters/{cluster}``.
        snapshot_id (str):
            Required. The ID by which the new snapshot should be
            referred to within the parent cluster, e.g., ``mysnapshot``
            of the form: ``[_a-zA-Z0-9][-_.a-zA-Z0-9]*`` rather than
            ``projects/{project}/instances/{instance}/clusters/{cluster}/snapshots/mysnapshot``.
        ttl (google.protobuf.duration_pb2.Duration):
            The amount of time that the new snapshot can
            stay active after it is created. Once 'ttl'
            expires, the snapshot will get deleted. The
            maximum amount of time a snapshot can stay
            active is 7 days. If 'ttl' is not specified, the
            default value of 24 hours will be used.
        description (str):
            Description of the snapshot.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cluster: str = proto.Field(
        proto.STRING,
        number=2,
    )
    snapshot_id: str = proto.Field(
        proto.STRING,
        number=3,
    )
    ttl: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=4,
        message=duration_pb2.Duration,
    )
    description: str = proto.Field(
        proto.STRING,
        number=5,
    )


class GetSnapshotRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.GetSnapshot][google.bigtable.admin.v2.BigtableTableAdmin.GetSnapshot]

    Note: This is a private alpha release of Cloud Bigtable snapshots.
    This feature is not currently available to most Cloud Bigtable
    customers. This feature might be changed in backward-incompatible
    ways and is not recommended for production use. It is not subject to
    any SLA or deprecation policy.

    Attributes:
        name (str):
            Required. The unique name of the requested snapshot. Values
            are of the form
            ``projects/{project}/instances/{instance}/clusters/{cluster}/snapshots/{snapshot}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListSnapshotsRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.ListSnapshots][google.bigtable.admin.v2.BigtableTableAdmin.ListSnapshots]

    Note: This is a private alpha release of Cloud Bigtable snapshots.
    This feature is not currently available to most Cloud Bigtable
    customers. This feature might be changed in backward-incompatible
    ways and is not recommended for production use. It is not subject to
    any SLA or deprecation policy.

    Attributes:
        parent (str):
            Required. The unique name of the cluster for which snapshots
            should be listed. Values are of the form
            ``projects/{project}/instances/{instance}/clusters/{cluster}``.
            Use ``{cluster} = '-'`` to list snapshots for all clusters
            in an instance, e.g.,
            ``projects/{project}/instances/{instance}/clusters/-``.
        page_size (int):
            The maximum number of snapshots to return per
            page. CURRENTLY UNIMPLEMENTED AND IGNORED.
        page_token (str):
            The value of ``next_page_token`` returned by a previous
            call.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListSnapshotsResponse(proto.Message):
    r"""Response message for
    [google.bigtable.admin.v2.BigtableTableAdmin.ListSnapshots][google.bigtable.admin.v2.BigtableTableAdmin.ListSnapshots]

    Note: This is a private alpha release of Cloud Bigtable snapshots.
    This feature is not currently available to most Cloud Bigtable
    customers. This feature might be changed in backward-incompatible
    ways and is not recommended for production use. It is not subject to
    any SLA or deprecation policy.

    Attributes:
        snapshots (MutableSequence[google.cloud.bigtable_admin_v2.types.Snapshot]):
            The snapshots present in the requested
            cluster.
        next_page_token (str):
            Set if not all snapshots could be returned in a single
            response. Pass this value to ``page_token`` in another
            request to get the next page of results.
    """

    @property
    def raw_page(self):
        return self

    snapshots: MutableSequence[gba_table.Snapshot] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_table.Snapshot,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class DeleteSnapshotRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.DeleteSnapshot][google.bigtable.admin.v2.BigtableTableAdmin.DeleteSnapshot]

    Note: This is a private alpha release of Cloud Bigtable snapshots.
    This feature is not currently available to most Cloud Bigtable
    customers. This feature might be changed in backward-incompatible
    ways and is not recommended for production use. It is not subject to
    any SLA or deprecation policy.

    Attributes:
        name (str):
            Required. The unique name of the snapshot to be deleted.
            Values are of the form
            ``projects/{project}/instances/{instance}/clusters/{cluster}/snapshots/{snapshot}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class SnapshotTableMetadata(proto.Message):
    r"""The metadata for the Operation returned by SnapshotTable.

    Note: This is a private alpha release of Cloud Bigtable
    snapshots. This feature is not currently available to most Cloud
    Bigtable customers. This feature might be changed in
    backward-incompatible ways and is not recommended for production
    use. It is not subject to any SLA or deprecation policy.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.SnapshotTableRequest):
            The request that prompted the initiation of
            this SnapshotTable operation.
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the original request was
            received.
        finish_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the operation failed or was
            completed successfully.
    """

    original_request: "SnapshotTableRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="SnapshotTableRequest",
    )
    request_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    finish_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class CreateTableFromSnapshotMetadata(proto.Message):
    r"""The metadata for the Operation returned by
    CreateTableFromSnapshot.
    Note: This is a private alpha release of Cloud Bigtable
    snapshots. This feature is not currently available to most Cloud
    Bigtable customers. This feature might be changed in
    backward-incompatible ways and is not recommended for production
    use. It is not subject to any SLA or deprecation policy.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.CreateTableFromSnapshotRequest):
            The request that prompted the initiation of
            this CreateTableFromSnapshot operation.
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the original request was
            received.
        finish_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the operation failed or was
            completed successfully.
    """

    original_request: "CreateTableFromSnapshotRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="CreateTableFromSnapshotRequest",
    )
    request_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    finish_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class CreateBackupRequest(proto.Message):
    r"""The request for
    [CreateBackup][google.bigtable.admin.v2.BigtableTableAdmin.CreateBackup].

    Attributes:
        parent (str):
            Required. This must be one of the clusters in the instance
            in which this table is located. The backup will be stored in
            this cluster. Values are of the form
            ``projects/{project}/instances/{instance}/clusters/{cluster}``.
        backup_id (str):
            Required. The id of the backup to be created. The
            ``backup_id`` along with the parent ``parent`` are combined
            as {parent}/backups/{backup_id} to create the full backup
            name, of the form:
            ``projects/{project}/instances/{instance}/clusters/{cluster}/backups/{backup_id}``.
            This string must be between 1 and 50 characters in length
            and match the regex [*a-zA-Z0-9][-*.a-zA-Z0-9]\*.
        backup (google.cloud.bigtable_admin_v2.types.Backup):
            Required. The backup to create.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    backup_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    backup: gba_table.Backup = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gba_table.Backup,
    )


class CreateBackupMetadata(proto.Message):
    r"""Metadata type for the operation returned by
    [CreateBackup][google.bigtable.admin.v2.BigtableTableAdmin.CreateBackup].

    Attributes:
        name (str):
            The name of the backup being created.
        source_table (str):
            The name of the table the backup is created
            from.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which this operation started.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            If set, the time at which this operation
            finished or was cancelled.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    source_table: str = proto.Field(
        proto.STRING,
        number=2,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )


class UpdateBackupRequest(proto.Message):
    r"""The request for
    [UpdateBackup][google.bigtable.admin.v2.BigtableTableAdmin.UpdateBackup].

    Attributes:
        backup (google.cloud.bigtable_admin_v2.types.Backup):
            Required. The backup to update. ``backup.name``, and the
            fields to be updated as specified by ``update_mask`` are
            required. Other fields are ignored. Update is only supported
            for the following fields:

            - ``backup.expire_time``.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. A mask specifying which fields (e.g.
            ``expire_time``) in the Backup resource should be updated.
            This mask is relative to the Backup resource, not to the
            request message. The field mask must always be specified;
            this prevents any future fields from being erased
            accidentally by clients that do not know about them.
    """

    backup: gba_table.Backup = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gba_table.Backup,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class GetBackupRequest(proto.Message):
    r"""The request for
    [GetBackup][google.bigtable.admin.v2.BigtableTableAdmin.GetBackup].

    Attributes:
        name (str):
            Required. Name of the backup. Values are of the form
            ``projects/{project}/instances/{instance}/clusters/{cluster}/backups/{backup}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class DeleteBackupRequest(proto.Message):
    r"""The request for
    [DeleteBackup][google.bigtable.admin.v2.BigtableTableAdmin.DeleteBackup].

    Attributes:
        name (str):
            Required. Name of the backup to delete. Values are of the
            form
            ``projects/{project}/instances/{instance}/clusters/{cluster}/backups/{backup}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListBackupsRequest(proto.Message):
    r"""The request for
    [ListBackups][google.bigtable.admin.v2.BigtableTableAdmin.ListBackups].

    Attributes:
        parent (str):
            Required. The cluster to list backups from. Values are of
            the form
            ``projects/{project}/instances/{instance}/clusters/{cluster}``.
            Use ``{cluster} = '-'`` to list backups for all clusters in
            an instance, e.g.,
            ``projects/{project}/instances/{instance}/clusters/-``.
        filter (str):
            A filter expression that filters backups listed in the
            response. The expression must specify the field name, a
            comparison operator, and the value that you want to use for
            filtering. The value must be a string, a number, or a
            boolean. The comparison operator must be <, >, <=, >=, !=,
            =, or :. Colon ':' represents a HAS operator which is
            roughly synonymous with equality. Filter rules are case
            insensitive.

            The fields eligible for filtering are:

            - ``name``
            - ``source_table``
            - ``state``
            - ``start_time`` (and values are of the format
              YYYY-MM-DDTHH:MM:SSZ)
            - ``end_time`` (and values are of the format
              YYYY-MM-DDTHH:MM:SSZ)
            - ``expire_time`` (and values are of the format
              YYYY-MM-DDTHH:MM:SSZ)
            - ``size_bytes``

            To filter on multiple expressions, provide each separate
            expression within parentheses. By default, each expression
            is an AND expression. However, you can include AND, OR, and
            NOT expressions explicitly.

            Some examples of using filters are:

            - ``name:"exact"`` --> The backup's name is the string
              "exact".
            - ``name:howl`` --> The backup's name contains the string
              "howl".
            - ``source_table:prod`` --> The source_table's name contains
              the string "prod".
            - ``state:CREATING`` --> The backup is pending creation.
            - ``state:READY`` --> The backup is fully created and ready
              for use.
            - ``(name:howl) AND (start_time < \"2018-03-28T14:50:00Z\")``
              --> The backup name contains the string "howl" and
              start_time of the backup is before 2018-03-28T14:50:00Z.
            - ``size_bytes > 10000000000`` --> The backup's size is
              greater than 10GB
        order_by (str):
            An expression for specifying the sort order of the results
            of the request. The string value should specify one or more
            fields in [Backup][google.bigtable.admin.v2.Backup]. The
            full syntax is described at https://aip.dev/132#ordering.

            Fields supported are:

            - name
            - source_table
            - expire_time
            - start_time
            - end_time
            - size_bytes
            - state

            For example, "start_time". The default sorting order is
            ascending. To specify descending order for the field, a
            suffix " desc" should be appended to the field name. For
            example, "start_time desc". Redundant space characters in
            the syntax are insigificant.

            If order_by is empty, results will be sorted by
            ``start_time`` in descending order starting from the most
            recently created backup.
        page_size (int):
            Number of backups to be returned in the
            response. If 0 or less, defaults to the server's
            maximum allowed page size.
        page_token (str):
            If non-empty, ``page_token`` should contain a
            [next_page_token][google.bigtable.admin.v2.ListBackupsResponse.next_page_token]
            from a previous
            [ListBackupsResponse][google.bigtable.admin.v2.ListBackupsResponse]
            to the same ``parent`` and with the same ``filter``.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=3,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=4,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListBackupsResponse(proto.Message):
    r"""The response for
    [ListBackups][google.bigtable.admin.v2.BigtableTableAdmin.ListBackups].

    Attributes:
        backups (MutableSequence[google.cloud.bigtable_admin_v2.types.Backup]):
            The list of matching backups.
        next_page_token (str):
            ``next_page_token`` can be sent in a subsequent
            [ListBackups][google.bigtable.admin.v2.BigtableTableAdmin.ListBackups]
            call to fetch more of the matching backups.
    """

    @property
    def raw_page(self):
        return self

    backups: MutableSequence[gba_table.Backup] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_table.Backup,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CopyBackupRequest(proto.Message):
    r"""The request for
    [CopyBackup][google.bigtable.admin.v2.BigtableTableAdmin.CopyBackup].

    Attributes:
        parent (str):
            Required. The name of the destination cluster that will
            contain the backup copy. The cluster must already exist.
            Values are of the form:
            ``projects/{project}/instances/{instance}/clusters/{cluster}``.
        backup_id (str):
            Required. The id of the new backup. The ``backup_id`` along
            with ``parent`` are combined as {parent}/backups/{backup_id}
            to create the full backup name, of the form:
            ``projects/{project}/instances/{instance}/clusters/{cluster}/backups/{backup_id}``.
            This string must be between 1 and 50 characters in length
            and match the regex [*a-zA-Z0-9][-*.a-zA-Z0-9]\*.
        source_backup (str):
            Required. The source backup to be copied from. The source
            backup needs to be in READY state for it to be copied.
            Copying a copied backup is not allowed. Once CopyBackup is
            in progress, the source backup cannot be deleted or cleaned
            up on expiration until CopyBackup is finished. Values are of
            the form:
            ``projects/<project>/instances/<instance>/clusters/<cluster>/backups/<backup>``.
        expire_time (google.protobuf.timestamp_pb2.Timestamp):
            Required. Required. The expiration time of the copied backup
            with microsecond granularity that must be at least 6 hours
            and at most 30 days from the time the request is received.
            Once the ``expire_time`` has passed, Cloud Bigtable will
            delete the backup and free the resources used by the backup.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    backup_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    source_backup: str = proto.Field(
        proto.STRING,
        number=3,
    )
    expire_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )


class CopyBackupMetadata(proto.Message):
    r"""Metadata type for the google.longrunning.Operation returned by
    [CopyBackup][google.bigtable.admin.v2.BigtableTableAdmin.CopyBackup].

    Attributes:
        name (str):
            The name of the backup being created through the copy
            operation. Values are of the form
            ``projects/<project>/instances/<instance>/clusters/<cluster>/backups/<backup>``.
        source_backup_info (google.cloud.bigtable_admin_v2.types.BackupInfo):
            Information about the source backup that is
            being copied from.
        progress (google.cloud.bigtable_admin_v2.types.OperationProgress):
            The progress of the
            [CopyBackup][google.bigtable.admin.v2.BigtableTableAdmin.CopyBackup]
            operation.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    source_backup_info: gba_table.BackupInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gba_table.BackupInfo,
    )
    progress: common.OperationProgress = proto.Field(
        proto.MESSAGE,
        number=3,
        message=common.OperationProgress,
    )


class CreateAuthorizedViewRequest(proto.Message):
    r"""The request for
    [CreateAuthorizedView][google.bigtable.admin.v2.BigtableTableAdmin.CreateAuthorizedView]

    Attributes:
        parent (str):
            Required. This is the name of the table the AuthorizedView
            belongs to. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
        authorized_view_id (str):
            Required. The id of the AuthorizedView to create. This
            AuthorizedView must not already exist. The
            ``authorized_view_id`` appended to ``parent`` forms the full
            AuthorizedView name of the form
            ``projects/{project}/instances/{instance}/tables/{table}/authorizedView/{authorized_view}``.
        authorized_view (google.cloud.bigtable_admin_v2.types.AuthorizedView):
            Required. The AuthorizedView to create.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    authorized_view_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    authorized_view: gba_table.AuthorizedView = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gba_table.AuthorizedView,
    )


class CreateAuthorizedViewMetadata(proto.Message):
    r"""The metadata for the Operation returned by
    CreateAuthorizedView.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.CreateAuthorizedViewRequest):
            The request that prompted the initiation of
            this CreateAuthorizedView operation.
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the original request was
            received.
        finish_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the operation failed or was
            completed successfully.
    """

    original_request: "CreateAuthorizedViewRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="CreateAuthorizedViewRequest",
    )
    request_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    finish_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class ListAuthorizedViewsRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.ListAuthorizedViews][google.bigtable.admin.v2.BigtableTableAdmin.ListAuthorizedViews]

    Attributes:
        parent (str):
            Required. The unique name of the table for which
            AuthorizedViews should be listed. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
        page_size (int):
            Optional. Maximum number of results per page.

            A page_size of zero lets the server choose the number of
            items to return. A page_size which is strictly positive will
            return at most that many items. A negative page_size will
            cause an error.

            Following the first request, subsequent paginated calls are
            not required to pass a page_size. If a page_size is set in
            subsequent calls, it must match the page_size given in the
            first request.
        page_token (str):
            Optional. The value of ``next_page_token`` returned by a
            previous call.
        view (google.cloud.bigtable_admin_v2.types.AuthorizedView.ResponseView):
            Optional. The resource_view to be applied to the returned
            AuthorizedViews' fields. Default to NAME_ONLY.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    view: gba_table.AuthorizedView.ResponseView = proto.Field(
        proto.ENUM,
        number=4,
        enum=gba_table.AuthorizedView.ResponseView,
    )


class ListAuthorizedViewsResponse(proto.Message):
    r"""Response message for
    [google.bigtable.admin.v2.BigtableTableAdmin.ListAuthorizedViews][google.bigtable.admin.v2.BigtableTableAdmin.ListAuthorizedViews]

    Attributes:
        authorized_views (MutableSequence[google.cloud.bigtable_admin_v2.types.AuthorizedView]):
            The AuthorizedViews present in the requested
            table.
        next_page_token (str):
            Set if not all tables could be returned in a single
            response. Pass this value to ``page_token`` in another
            request to get the next page of results.
    """

    @property
    def raw_page(self):
        return self

    authorized_views: MutableSequence[gba_table.AuthorizedView] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_table.AuthorizedView,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GetAuthorizedViewRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.GetAuthorizedView][google.bigtable.admin.v2.BigtableTableAdmin.GetAuthorizedView]

    Attributes:
        name (str):
            Required. The unique name of the requested AuthorizedView.
            Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}/authorizedViews/{authorized_view}``.
        view (google.cloud.bigtable_admin_v2.types.AuthorizedView.ResponseView):
            Optional. The resource_view to be applied to the returned
            AuthorizedView's fields. Default to BASIC.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    view: gba_table.AuthorizedView.ResponseView = proto.Field(
        proto.ENUM,
        number=2,
        enum=gba_table.AuthorizedView.ResponseView,
    )


class UpdateAuthorizedViewRequest(proto.Message):
    r"""The request for
    [UpdateAuthorizedView][google.bigtable.admin.v2.BigtableTableAdmin.UpdateAuthorizedView].

    Attributes:
        authorized_view (google.cloud.bigtable_admin_v2.types.AuthorizedView):
            Required. The AuthorizedView to update. The ``name`` in
            ``authorized_view`` is used to identify the AuthorizedView.
            AuthorizedView name must in this format:
            ``projects/{project}/instances/{instance}/tables/{table}/authorizedViews/{authorized_view}``.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. The list of fields to update. A mask specifying
            which fields in the AuthorizedView resource should be
            updated. This mask is relative to the AuthorizedView
            resource, not to the request message. A field will be
            overwritten if it is in the mask. If empty, all fields set
            in the request will be overwritten. A special value ``*``
            means to overwrite all fields (including fields not set in
            the request).
        ignore_warnings (bool):
            Optional. If true, ignore the safety checks
            when updating the AuthorizedView.
    """

    authorized_view: gba_table.AuthorizedView = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gba_table.AuthorizedView,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    ignore_warnings: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class UpdateAuthorizedViewMetadata(proto.Message):
    r"""Metadata for the google.longrunning.Operation returned by
    [UpdateAuthorizedView][google.bigtable.admin.v2.BigtableTableAdmin.UpdateAuthorizedView].

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.UpdateAuthorizedViewRequest):
            The request that prompted the initiation of
            this UpdateAuthorizedView operation.
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the original request was
            received.
        finish_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the operation failed or was
            completed successfully.
    """

    original_request: "UpdateAuthorizedViewRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="UpdateAuthorizedViewRequest",
    )
    request_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    finish_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class DeleteAuthorizedViewRequest(proto.Message):
    r"""Request message for
    [google.bigtable.admin.v2.BigtableTableAdmin.DeleteAuthorizedView][google.bigtable.admin.v2.BigtableTableAdmin.DeleteAuthorizedView]

    Attributes:
        name (str):
            Required. The unique name of the AuthorizedView to be
            deleted. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}/authorizedViews/{authorized_view}``.
        etag (str):
            Optional. The current etag of the
            AuthorizedView. If an etag is provided and does
            not match the current etag of the
            AuthorizedView, deletion will be blocked and an
            ABORTED error will be returned.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    etag: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CreateSchemaBundleRequest(proto.Message):
    r"""The request for
    [CreateSchemaBundle][google.bigtable.admin.v2.BigtableTableAdmin.CreateSchemaBundle].

    Attributes:
        parent (str):
            Required. The parent resource where this schema bundle will
            be created. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
        schema_bundle_id (str):
            Required. The unique ID to use for the schema
            bundle, which will become the final component of
            the schema bundle's resource name.
        schema_bundle (google.cloud.bigtable_admin_v2.types.SchemaBundle):
            Required. The schema bundle to create.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    schema_bundle_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    schema_bundle: gba_table.SchemaBundle = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gba_table.SchemaBundle,
    )


class CreateSchemaBundleMetadata(proto.Message):
    r"""The metadata for the Operation returned by
    [CreateSchemaBundle][google.bigtable.admin.v2.BigtableTableAdmin.CreateSchemaBundle].

    Attributes:
        name (str):
            The unique name identifying this schema bundle. Values are
            of the form
            ``projects/{project}/instances/{instance}/tables/{table}/schemaBundles/{schema_bundle}``
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which this operation started.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            If set, the time at which this operation
            finished or was canceled.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class UpdateSchemaBundleRequest(proto.Message):
    r"""The request for
    [UpdateSchemaBundle][google.bigtable.admin.v2.BigtableTableAdmin.UpdateSchemaBundle].

    Attributes:
        schema_bundle (google.cloud.bigtable_admin_v2.types.SchemaBundle):
            Required. The schema bundle to update.

            The schema bundle's ``name`` field is used to identify the
            schema bundle to update. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}/schemaBundles/{schema_bundle}``
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. The list of fields to update.
        ignore_warnings (bool):
            Optional. If set, ignore the safety checks
            when updating the Schema Bundle. The safety
            checks are:

            - The new Schema Bundle is backwards compatible
              with the existing Schema Bundle.
    """

    schema_bundle: gba_table.SchemaBundle = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gba_table.SchemaBundle,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    ignore_warnings: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class UpdateSchemaBundleMetadata(proto.Message):
    r"""The metadata for the Operation returned by
    [UpdateSchemaBundle][google.bigtable.admin.v2.BigtableTableAdmin.UpdateSchemaBundle].

    Attributes:
        name (str):
            The unique name identifying this schema bundle. Values are
            of the form
            ``projects/{project}/instances/{instance}/tables/{table}/schemaBundles/{schema_bundle}``
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which this operation started.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            If set, the time at which this operation
            finished or was canceled.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class GetSchemaBundleRequest(proto.Message):
    r"""The request for
    [GetSchemaBundle][google.bigtable.admin.v2.BigtableTableAdmin.GetSchemaBundle].

    Attributes:
        name (str):
            Required. The unique name of the schema bundle to retrieve.
            Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}/schemaBundles/{schema_bundle}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListSchemaBundlesRequest(proto.Message):
    r"""The request for
    [ListSchemaBundles][google.bigtable.admin.v2.BigtableTableAdmin.ListSchemaBundles].

    Attributes:
        parent (str):
            Required. The parent, which owns this collection of schema
            bundles. Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}``.
        page_size (int):
            The maximum number of schema bundles to
            return. If the value is positive, the server may
            return at most this value. If unspecified, the
            server will return the maximum allowed page
            size.
        page_token (str):
            A page token, received from a previous ``ListSchemaBundles``
            call. Provide this to retrieve the subsequent page.

            When paginating, all other parameters provided to
            ``ListSchemaBundles`` must match the call that provided the
            page token.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListSchemaBundlesResponse(proto.Message):
    r"""The response for
    [ListSchemaBundles][google.bigtable.admin.v2.BigtableTableAdmin.ListSchemaBundles].

    Attributes:
        schema_bundles (MutableSequence[google.cloud.bigtable_admin_v2.types.SchemaBundle]):
            The schema bundles from the specified table.
        next_page_token (str):
            A token, which can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    schema_bundles: MutableSequence[gba_table.SchemaBundle] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_table.SchemaBundle,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class DeleteSchemaBundleRequest(proto.Message):
    r"""The request for
    [DeleteSchemaBundle][google.bigtable.admin.v2.BigtableTableAdmin.DeleteSchemaBundle].

    Attributes:
        name (str):
            Required. The unique name of the schema bundle to delete.
            Values are of the form
            ``projects/{project}/instances/{instance}/tables/{table}/schemaBundles/{schema_bundle}``
        etag (str):
            Optional. The etag of the schema bundle.
            If this is provided, it must match the server's
            etag. The server returns an ABORTED error on a
            mismatched etag.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    etag: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
